@@ -34,9 +34,8 @@ static void add_fresh_update_toggled_cb (GtkToggleButton *button, Add_dialog_dat
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->update), FALSE);
 }
 
-static void password_toggled_cb (GtkButton *button, gpointer user_data)
+static void password_toggled_cb (GtkToggleButton *button, Add_dialog_data *add_dialog)
 {
-	Add_dialog_data *add_dialog = user_data;
 	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(add_dialog->add_password)) )
 	{
 		gtk_widget_set_sensitive (add_dialog->add_password_entry, TRUE);
@@ -476,7 +475,7 @@ void xa_execute_add_commands (XArchive *archive, GSList *list, gboolean recurse,
 	g_slist_free(dirlist);
 
 	archive->status = XARCHIVESTATUS_ADD;
-	(*archive->add)(archive, files, compression);
+	(*archive->archiver->add)(archive, files, compression);
 
 	g_free(archive->child_dir);
 	archive->child_dir = NULL;
