@@ -16,19 +16,20 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef __ADD_DIALOG_H
-#define __ADD_DIALOG_H
+#ifndef XARCHIVER_ADD_DIALOG_H
+#define XARCHIVER_ADD_DIALOG_H
 
+#include <gtk/gtk.h>
 #include "archive.h"
 
-typedef struct
+typedef struct Add_dialog_data
 {
 	GtkWidget *dialog1;
-	GtkWidget *dialog_vbox1;
 	GtkWidget *alignment2;
 	GtkWidget *notebook1;
 	GtkWidget *filechooserwidget1;
 	GtkWidget *frame1;
+	GtkWidget *label;
 	GtkWidget *store_path;
 	GtkWidget *no_store_path;
 	GtkWidget *option_notebook_vbox;
@@ -45,20 +46,12 @@ typedef struct
 	GtkWidget *add_label;
 	GtkWidget *cancel_button;
 	GtkWidget *add_button;
-	GtkTooltips *add_option_tooltip;
 	GtkWidget *add_option_label;
-	GtkTooltips *option_tooltip;
-	GtkObject *compression_value;
 } Add_dialog_data;
 
 Add_dialog_data *xa_create_add_dialog();
-void xa_set_add_dialog_options(Add_dialog_data *,XArchive *);
-void add_fresh_update_toggled_cb (GtkToggleButton *, Add_dialog_data *);
-void add_update_fresh_toggled_cb (GtkToggleButton *, Add_dialog_data *);
-void password_toggled_cb ( GtkButton* ,gpointer);
-void xa_parse_add_dialog_options ( XArchive *, Add_dialog_data *);
-void xa_select_files_to_add ( GtkButton* ,gpointer );
-void xa_execute_add_commands (XArchive *archive,GSList *list,gchar *compression_string);
-void fix_adjustment_value (GtkAdjustment *,gpointer );
-#endif
+void xa_execute_add_commands(XArchive *, GSList *, gboolean, gchar *);
+void xa_parse_add_dialog_options(XArchive *, Add_dialog_data *);
+void xa_set_add_dialog_options(Add_dialog_data *, XArchive *);
 
+#endif
