@@ -255,9 +255,8 @@ static void xa_browse_dir_sidebar (XEntry *entry, GtkTreeStore *model, gchar *pa
 
 }
 
-static gboolean _xa_dir_sidebar_select_row (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
+static gboolean xa_dir_sidebar_find_row (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer entry)
 {
-	XEntry *entry = data;
 	XEntry *entry2;
 	GtkTreeIter parent;
 	gboolean value;
@@ -295,6 +294,10 @@ gboolean xa_get_compressed_tar_type (XArchiveType *type)
 			*type = XARCHIVETYPE_TAR_GZ;
 			break;
 
+		case XARCHIVETYPE_LRZIP:
+			*type = XARCHIVETYPE_TAR_LRZ;
+			break;
+
 		case XARCHIVETYPE_LZ4:
 			*type = XARCHIVETYPE_TAR_LZ4;
 			break;
@@ -315,6 +318,10 @@ gboolean xa_get_compressed_tar_type (XArchiveType *type)
 			*type = XARCHIVETYPE_TAR_XZ;
 			break;
 
+		case XARCHIVETYPE_ZSTD:
+			*type = XARCHIVETYPE_TAR_ZSTD;
+			break;
+
 		default:
 			return FALSE;
 	}
@@ -333,7 +340,7 @@ XArchive *xa_init_archive_structure (ArchiveType xa)
 		return NULL;
 
 	archive->type = xa.type;
-	archive->version = xa.version;
+	archive->tag = xa.tag;
 
 	entry = g_new0(XEntry, 1);
 
@@ -346,14 +353,9 @@ XArchive *xa_init_archive_structure (ArchiveType xa)
 	entry->filename = "";
 	archive->root_entry = entry;
 
-	archive->ask = ask[xa.type];
-	archive->list = list[xa.type];
-	archive->test = test[xa.type];
-	archive->extract = extract[xa.type];
-	archive->add = add[xa.type];
-	archive->delete = delete[xa.type];
+	archive->archiver = &archiver[xa.type];
 
-	(*archive->ask)(archive);
+	(*archive->archiver->ask)(archive);
 
 	return archive;
 }
@@ -508,6 +510,50 @@ gboolean xa_create_working_directory (XArchive *archive)
 	return TRUE;
 }
 
+gchar *xa_create_containing_directory (XArchive *archive, const gchar *path)
+{
+	gchar *stem, *dir;
+	char *dot;
+	guint i;
+
+	stem = g_path_get_basename(archive->path[0]);
+	dot = strrchr(stem, '.');
+
+	if (dot)
+		*dot = 0;
+
+	if (g_str_has_suffix(stem, ".tar"))
+	{
+		dot = strrchr(stem, '.');
+		*dot = 0;
+	}
+
+	dir = g_strconcat(path, "/", stem, NULL);
+
+	for (i = 0; i < 100; i++)
+	{
+		if (i > 0)
+		{
+			g_free(dir);
+			dir = g_strdup_printf("%s/%s-(%d)", path, stem, i);
+		}
+
+		if (!g_file_test(dir, G_FILE_TEST_EXISTS))
+		{
+			if (g_mkdir_with_parents(dir, 0700) == 0)
+			{
+				g_free(stem);
+
+				return dir;
+			}
+		}
+	}
+
+	g_free(stem);
+
+	return NULL;
+}
+
 gboolean xa_run_command (XArchive *archive, const gchar *command)
 {
 	pid_t pid = 0;
@@ -542,15 +588,17 @@ gboolean xa_run_command (XArchive *archive, const gchar *command)
 
 gint xa_find_archive_index (gint page_num)
 {
-	GtkWidget *scrollwindow;
+	GtkWidget *page;
 	gint i;
 
-	scrollwindow = gtk_notebook_get_nth_page(notebook, page_num);
+	page = gtk_notebook_get_nth_page(notebook, page_num);
+
 	for (i = 0; i < MAX_XARCHIVES; i++)
 	{
-		if (archive[i] != NULL && archive[i]->page == scrollwindow)
+		if (archive[i] != NULL && archive[i]->page == page)
 			return i;
 	}
+
 	return -1;
 }
 
@@ -612,10 +660,6 @@ XEntry *xa_set_archive_entries_for_each_row (XArchive *archive, const gchar *fil
 	gchar **components;
 	guint n = 0;
 
-	/* check for batch mode */
-	if (!xa_main_window)
-		return NULL;
-
 	components = g_strsplit(filename, "/", -1);
 
 	if (*filename == '/')
@@ -718,7 +762,7 @@ void xa_fill_list_with_recursed_entries(XEntry *entry,GSList **p_file_list)
 void xa_detect_encrypted_archive (XArchive *archive)
 {
 	archive->status = XARCHIVESTATUS_LIST;
-	(*archive->list)(archive);
+	(*archive->archiver->list)(archive);
 
 	do
 	{
@@ -749,7 +793,7 @@ void xa_fill_dir_sidebar(XArchive *archive,gboolean force_reload)
 	}
 }
 
-void xa_dir_sidebar_row_selected (GtkTreeSelection *selection, gpointer data)
+void xa_dir_sidebar_row_selected (GtkTreeSelection *selection, gpointer user_data)
 {
 	XEntry *entry;
 	GtkTreeIter iter;
@@ -758,11 +802,9 @@ void xa_dir_sidebar_row_selected (GtkTreeSelection *selection, gpointer data)
 	GtkTreeModel *model;
 	GString *string = g_string_new("");
 	gchar *dir;
-	gint current_page;
 	gint idx;
 
-	current_page = gtk_notebook_get_current_page(notebook);
-	idx = xa_find_archive_index(current_page);
+	idx = xa_find_archive_index(gtk_notebook_get_current_page(notebook));
 
 	if ((idx >= 0) && gtk_tree_selection_get_selected(selection, &model, &iter))
 	{
@@ -793,13 +835,12 @@ void xa_dir_sidebar_row_selected (GtkTreeSelection *selection, gpointer data)
 
 void xa_dir_sidebar_select_row (XEntry *entry)
 {
-	gtk_tree_model_foreach(GTK_TREE_MODEL(archive_dir_model), (GtkTreeModelForeachFunc) _xa_dir_sidebar_select_row, entry);
+	gtk_tree_model_foreach(GTK_TREE_MODEL(archive_dir_model), xa_dir_sidebar_find_row, entry);
 }
 
-gint xa_sort_dirs_before_files(GtkTreeModel *model,GtkTreeIter *a,GtkTreeIter *b,gpointer data)
+gint xa_sort_dirs_before_files (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, XArchive *archive)
 {
 	XEntry *entry1, *entry2;
-	XArchive *archive = data;
 
 	gtk_tree_model_get(model, a, archive->columns - 1, &entry1, -1);
 	gtk_tree_model_get(model, b, archive->columns - 1, &entry2, -1);
