@@ -31,6 +31,7 @@ typedef enum
 	XARCHIVETYPE_DEB,
 	XARCHIVETYPE_GZIP,
 	XARCHIVETYPE_LHA,
+	XARCHIVETYPE_LRZIP,
 	XARCHIVETYPE_LZ4,
 	XARCHIVETYPE_LZIP,
 	XARCHIVETYPE_LZMA,
@@ -40,14 +41,17 @@ typedef enum
 	XARCHIVETYPE_TAR,
 	XARCHIVETYPE_TAR_BZ2,
 	XARCHIVETYPE_TAR_GZ,
+	XARCHIVETYPE_TAR_LRZ,
 	XARCHIVETYPE_TAR_LZ,
 	XARCHIVETYPE_TAR_LZ4,
 	XARCHIVETYPE_TAR_LZMA,
 	XARCHIVETYPE_TAR_LZOP,
 	XARCHIVETYPE_TAR_XZ,
 	XARCHIVETYPE_TAR_Z,
+	XARCHIVETYPE_TAR_ZSTD,
 	XARCHIVETYPE_XZ,
 	XARCHIVETYPE_ZIP,
+	XARCHIVETYPE_ZSTD,
 	XARCHIVETYPE_TYPES,
 	XARCHIVETYPE_UNKNOWN,
 	XARCHIVETYPE_NOT_FOUND
@@ -74,19 +78,35 @@ typedef enum
 	XA_CHILD_PROCS
 } XAChildProcess;
 
+typedef struct _XArchive XArchive;
+
+typedef void (*ask_func)(XArchive *);
+typedef void (*list_func)(XArchive *);
+typedef void (*parse_output_func)(gchar *, XArchive *);
+typedef void (*test_func)(XArchive *);
+typedef gboolean (*extract_func)(XArchive *, GSList *);
+typedef void (*add_func)(XArchive *, GSList *, gchar *);
+typedef void (*delete_func)(XArchive *, GSList *);
+
 typedef struct
 {
 	gchar *program[2];
 	gboolean is_compressor;
 	GSList *type;
 	GSList *glob;
-	GSList *version;
+	GSList *tags;
+	ask_func ask;
+	list_func list;
+	test_func test;
+	extract_func extract;
+	add_func add;
+	delete_func delete;
 } XArchiver;
 
 typedef struct
 {
 	XArchiveType type;
-	gushort version;
+	gushort tag;
 } ArchiveType;
 
 typedef struct _XEntry XEntry;
@@ -102,23 +122,13 @@ struct _XEntry
 	XEntry *next;
 };
 
-typedef struct _XArchive XArchive;
-
-typedef void (*ask_func)(XArchive *);
-typedef void (*list_func)(XArchive *);
-typedef void (*parse_output_func)(gchar *, XArchive *);
-typedef void (*test_func)(XArchive *);
-typedef gboolean (*extract_func)(XArchive *, GSList *);
-typedef void (*add_func)(XArchive *, GSList *, gchar *);
-typedef void (*delete_func)(XArchive *, GSList *);
-
 typedef struct _XAClipboard XAClipboard;
 
 struct _XArchive
 {
 	/* characteristics */
 	XArchiveType type;
-	gushort version;
+	gushort tag;
 	XArchiveStatus status;
 	/* data */
 	XEntry *root_entry;
@@ -180,13 +190,8 @@ struct _XArchive
 	gint child_fderr;
 	GSList *output;
 	/* (un)compressor interface */
-	ask_func ask;
-	list_func list;
+	XArchiver *archiver;
 	parse_output_func parse_output;
-	test_func test;
-	extract_func extract;
-	add_func add;
-	delete_func delete;
 };
 
 extern XArchive *archive[];
@@ -211,6 +216,7 @@ struct _XAClipboard
 
 gchar *xa_build_full_path_name_from_entry(XEntry *);
 void xa_clean_archive_structure(XArchive *);
+gchar *xa_create_containing_directory(XArchive *, const gchar *);
 gboolean xa_create_working_directory(XArchive *);
 void xa_detect_encrypted_archive(XArchive *);
 void xa_fill_dir_sidebar(XArchive *, gboolean);
@@ -225,7 +231,7 @@ gboolean xa_run_command(XArchive *, const gchar *);
 XEntry *xa_set_archive_entries_for_each_row(XArchive *, const gchar *, gpointer *);
 void xa_dir_sidebar_row_selected(GtkTreeSelection *, gpointer);
 void xa_dir_sidebar_select_row(XEntry *);
-gint xa_sort_dirs_before_files(GtkTreeModel *, GtkTreeIter *, GtkTreeIter *, gpointer);
+gint xa_sort_dirs_before_files(GtkTreeModel *, GtkTreeIter *, GtkTreeIter *, XArchive *);
 void xa_spawn_async_process(XArchive *, const gchar *);
 
 #endif
