@@ -56,7 +56,6 @@ static void xa_arj_parse_output (gchar *line, XArchive *archive)
 	XEntry *entry;
 	gpointer item[7];
 	unsigned int linesize,n,a;
-	guint64 file_size;
 	static gchar *filename;
 	gboolean unarj, lfn, dir, encrypted;
 
@@ -110,7 +109,6 @@ static void xa_arj_parse_output (gchar *line, XArchive *archive)
 			filename = g_strdup(line + 5);
 		}
 
-		archive->files++;
 		fname_line = TRUE;
 		if (lfn)
 			return;
@@ -125,7 +123,6 @@ static void xa_arj_parse_output (gchar *line, XArchive *archive)
 		for(; n < linesize && line[n] != ' '; n++);
 		line[n]='\0';
 		item[0] = line + a;
-		file_size = g_ascii_strtoull(item[0], NULL, 0);
 		n++;
 
 		/* Compressed */
@@ -148,42 +145,50 @@ static void xa_arj_parse_output (gchar *line, XArchive *archive)
 		line[58] = '\0';
 		item[4] = line + 50;
 
+		/* CRC */
+		if (unarj)
+		{
+			line[67] = '\0';
+			item[6] = line + 59;
+		}
+
 		/* Permissions */
 		line[unarj ? 72 : 69] = '\0';
 		item[5] = line + (unarj ? 68 : 59);
 
-		dir = (line[73] == 'D');   // for unarj only
-
-		/* GUA */
-		line[73] = '\0';
-		item[6] = (unarj ? "" : line + 70);
-
-		/* BPMGS */
+		/* BTPMGVX */
 		if (unarj)
+		{
+			dir = (line[73] == 'D');
 			encrypted = (line[76] == 'G');
+		}
+		/* BPMGS */
 		else
+		{
+			dir = (line[59] == 'd');
 			encrypted = (line[77] == '1');
+		}
+
 		if (encrypted)
 			archive->has_password = TRUE;
 
 		if (unarj && dir)
-		{
 			/* skip entry since unarj lacks directory structure information */
-			archive->files--;
 			entry = NULL;
-		}
 		else
-		{
-			archive->files_size += file_size;
-			entry = xa_set_archive_entries_for_each_row (archive,filename,item);
-		}
+			entry = xa_set_archive_entries_for_each_row(archive, filename, item);
 
-		if (entry != NULL)
+		if (entry)
 		{
-			if (line[59] == 'd')      // for arj only
+			if (dir)
 				entry->is_dir = TRUE;
 
 			entry->is_encrypted	= encrypted;
+
+			if (!entry->is_dir)
+				archive->files++;
+
+			archive->files_size += g_ascii_strtoull(item[0], NULL, 0);
 		}
 
 		g_free(filename);
@@ -193,8 +198,8 @@ static void xa_arj_parse_output (gchar *line, XArchive *archive)
 
 void xa_arj_list (XArchive *archive)
 {
-	const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
-	const gchar *titles[] = {_("Original"), _("Compressed"), _("Ratio"), _("Date"), _("Time"), _("Attributes"), _("GUA")};
+	const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, archiver[archive->type].is_compressor ? G_TYPE_POINTER : G_TYPE_STRING, G_TYPE_POINTER};
+	const gchar *titles[] = {_("Original Size"), _("Compressed"), _("Ratio"), _("Date"), _("Time"), archiver[archive->type].is_compressor ? _("Permissions") : _("Attributes"), archiver[archive->type].is_compressor ? NULL : _("Checksum")};
 	guint i;
 
 	data_line = FALSE;
@@ -206,7 +211,8 @@ void xa_arj_list (XArchive *archive)
 	xa_spawn_async_process (archive,command);
 	g_free (command);
 
-	archive->columns = 10;
+	archive->columns = (archiver[archive->type].is_compressor ? 9 : 10);
+	archive->size_column = 2;
 	archive->column_types = g_malloc0(sizeof(types));
 
 	for (i = 0; i < archive->columns; i++)
@@ -309,7 +315,7 @@ void xa_arj_add (XArchive *archive, GSList *file_list, gchar *compression)
 	                      archive->do_freshen ? " -f" : "",
 	                      archive->do_move ? " -d1" : "",
 	                      " -m", compression,
-	                      password_str, " -i -y ",
+	                      password_str, " -2d -i -y ",
 	                      archive->path[1], files->str, NULL);
 	g_free(password_str);
 	g_string_free(files,TRUE);
