@@ -32,6 +32,7 @@ void xa_deb_ask (XArchive *archive)
 
 static void xa_ar_parse_output (gchar *line, XArchive *archive)
 {
+	XEntry *entry;
 	gchar *filename;
 	gpointer item[4];
 	gint n = 0, a = 0 ,linesize = 0;
@@ -40,14 +41,14 @@ static void xa_ar_parse_output (gchar *line, XArchive *archive)
 
 	/* Permissions */
 	line[9] = '\0';
-	item[0] = line;
+	item[2] = line;
 
 	/* Owner */
 	for(n=12; n < linesize; ++n)
 		if(line[n] == ' ')
 			break;
 	line[n] = '\0';
-	item[1] = line+10;
+	item[3] = line+10;
 
 	/* Size */
 	for(++n; n < linesize; ++n)
@@ -60,8 +61,7 @@ static void xa_ar_parse_output (gchar *line, XArchive *archive)
 			break;
 
 	line[n] = '\0';
-	item[2] = line + a;
-	archive->files_size += g_ascii_strtoull(item[2],NULL,0);
+	item[0] = line + a;
 	a = ++n;
 
 	/* Date Modified */
@@ -79,21 +79,30 @@ static void xa_ar_parse_output (gchar *line, XArchive *archive)
 		}
 	}
 	line[n] = '\0';
-	item[3] = line + a;
+	item[1] = line + a;
 
 	n++;
 	line[linesize-1] = '\0';
 
-	archive->files++;
 	filename = g_strdup(line + n);
-	xa_set_archive_entries_for_each_row (archive,filename,item);
+
+	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+
+	if (entry)
+	{
+		if (!entry->is_dir)
+			archive->files++;
+
+		archive->files_size += g_ascii_strtoull(item[0], NULL, 0);
+	}
+
 	g_free(filename);
 }
 
 void xa_deb_list (XArchive *archive)
 {
-	const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_POINTER};
-	const gchar *titles[] = {_("Permissions"), _("Owner/Group"), _("Size"), _("Date modified")};
+	const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
+	const gchar *titles[] = {_("Original Size"), _("Date and Time"), _("Permissions"), _("UID/GID")};
 	gchar *command = NULL;
 	guint i;
 
@@ -105,6 +114,7 @@ void xa_deb_list (XArchive *archive)
 	g_free (command);
 
 	archive->columns = 7;
+	archive->size_column = 2;
 	archive->column_types = g_malloc0(sizeof(types));
 
 	for (i = 0; i < archive->columns; i++)
@@ -120,7 +130,7 @@ gboolean xa_deb_extract (XArchive *archive, GSList *file_list)
 	gboolean result;
 
 	files = xa_quote_filenames(file_list, NULL, TRUE);
-	chdir(archive->extraction_dir);
+	archive->child_dir = g_strdup(archive->extraction_dir);
 	command = g_strconcat(archiver[archive->type].program[0], " x",
 	                      archive->do_touch ? " " : "o ",
 	                      archive->path[1], files->str, NULL);
@@ -129,5 +139,8 @@ gboolean xa_deb_extract (XArchive *archive, GSList *file_list)
 	result = xa_run_command(archive, command);
 	g_free(command);
 
+	g_free(archive->child_dir);
+	archive->child_dir = NULL;
+
 	return result;
 }
