@@ -69,10 +69,13 @@ static void xa_select_where_to_extract (GtkEntry *entry, gint icon_pos, GTK_COMP
 	gtk_widget_destroy(file_selector);
 }
 
-static void xa_activate_entry (GtkToggleButton *button, gpointer data)
+static void xa_toggle_all_files_radio (GtkToggleButton *button, Extract_dialog_data *dialog)
 {
-	Extract_dialog_data *dialog = data;
+	gtk_widget_set_sensitive(dialog->ensure_directory, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->all_files_radio)));
+}
 
+static void xa_activate_entry (GtkToggleButton *button, Extract_dialog_data *dialog)
+{
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(dialog->files_radio)))
 	{
 		gtk_widget_set_sensitive (dialog->entry2,TRUE);
@@ -87,25 +90,24 @@ static void xa_multi_extract_dialog_selection_changed (GtkTreeSelection *selecti
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	XArchiveType type;
-	guint version;
+	guint tag;
 	XArchive archive = {0};
 
 	if (gtk_tree_selection_get_selected (selection,&model,&iter))
 	{
-		gtk_tree_model_get(model, &iter, 3, &type, 4, &version, -1);
+		gtk_tree_model_get(model, &iter, 3, &type, 4, &tag, -1);
 
 		archive.type = type;
-		archive.version = version;
+		archive.tag = tag;
 
-		(*ask[type])(&archive);
+		(*archiver[type].ask)(&archive);
 
 		gtk_widget_set_sensitive(dialog_data->full_path, archive.can_full_path[0]);
 	}
 }
 
-static void xa_multi_extract_dialog_drag_data_received (GtkWidget *widget, GdkDragContext *context, int x, int y, GtkSelectionData *data, unsigned int info, unsigned int time, gpointer user_data)
+static void xa_multi_extract_dialog_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *data, guint info, guint time, gpointer dialog_data)
 {
-	Multi_extract_data *dialog_data = user_data;
 	gchar **array = NULL;
 	gchar *filename;
 	unsigned int len = 0;
@@ -255,7 +257,7 @@ static gchar *xa_multi_extract_one_archive (Multi_extract_data *dialog, gchar *f
 	if (!error)
 	{
 		archive->status = XARCHIVESTATUS_EXTRACT;
-		(*archive->extract)(archive,NULL);
+		(*archive->archiver->extract)(archive,NULL);
 	}
 
 	/* return from batch mode */
@@ -288,7 +290,7 @@ Extract_dialog_data *xa_create_extract_dialog()
 	GTK_COMPAT_TOOLTIPS;
 	GSList *radiobutton1_group;
 	Extract_dialog_data *dialog_data;
-	GtkWidget *hbox1, *hbox2, *hbox3, *vbox1, *vbox2, *vbox3, *vbox5, *alignment1, *alignment2, *alignment3, *label1, *label2, *label3;
+	GtkWidget *hbox1, *hbox2, *hbox3, *vbox1, *vbox2, *vbox3, *vbox5, *alignment, *alignment1, *alignment2, *alignment3, *label1, *label2, *label3;
 	GtkWidget *frame1, *frame2, *dialog_action_area1;
 	GtkWidget *cancel_button, *extract_button, *extract_image, *extract_hbox, *extract_label;
 
@@ -313,6 +315,14 @@ Extract_dialog_data *xa_create_extract_dialog()
 	GTK_COMPAT_ENTRY_ICON(dialog_data->destination_path_entry, xa_select_where_to_extract, dialog_data);
 	gtk_box_pack_start (GTK_BOX (vbox1),dialog_data->destination_path_entry,FALSE,FALSE,0);
 
+	alignment = gtk_alignment_new(0.5, 0.5, 1, 1);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 14, 0);
+
+	dialog_data->ensure_directory = gtk_check_button_new_with_mnemonic(_("Ensure a containing directory"));
+	gtk_widget_set_tooltip_text(dialog_data->ensure_directory, _("Ensure that the contents of the extracted archive is always in a containing directory"));
+	gtk_container_add(GTK_CONTAINER(alignment), dialog_data->ensure_directory);
+	gtk_box_pack_start(GTK_BOX(vbox1), alignment, FALSE, FALSE, 0);
+
 	hbox1 = gtk_hbox_new (TRUE,10);
 	gtk_box_pack_start (GTK_BOX (vbox1),hbox1,TRUE,TRUE,0);
 
@@ -333,6 +343,7 @@ Extract_dialog_data *xa_create_extract_dialog()
 	dialog_data->all_files_radio = gtk_radio_button_new_with_mnemonic (NULL,_("All files"));
 	gtk_box_pack_start (GTK_BOX (vbox3),dialog_data->all_files_radio,FALSE,FALSE,0);
 	radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (dialog_data->all_files_radio));
+	g_signal_connect(dialog_data->all_files_radio, "toggled", G_CALLBACK(xa_toggle_all_files_radio), dialog_data);
 
 	dialog_data->selected_radio = gtk_radio_button_new_with_mnemonic(radiobutton1_group, _("Selected files"));
 	gtk_box_pack_start (GTK_BOX (vbox3),dialog_data->selected_radio,FALSE,FALSE,0);
@@ -511,6 +522,8 @@ void xa_parse_extract_dialog_options (XArchive *archive,Extract_dialog_data *dia
 				extraction_dir = g_strconcat(cur_dir, "/", destination_path, NULL);
 				g_free(archive->extraction_dir);
 				archive->extraction_dir = xa_escape_bad_chars(extraction_dir, ESCAPES);
+				g_free(destination_path);
+				destination_path = g_strdup(extraction_dir);
 				g_free(extraction_dir);
 				g_free (cur_dir);
 			}
@@ -537,6 +550,43 @@ void xa_parse_extract_dialog_options (XArchive *archive,Extract_dialog_data *dia
 				break;
 			}
 			done = TRUE;
+
+			if (gtk_widget_is_sensitive(dialog_data->ensure_directory) &&
+			    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->ensure_directory)))
+			{
+				XEntry *entry;
+				guint i = 0;
+
+				entry = archive->root_entry->child;
+
+				while (entry)
+				{
+					if (++i > 1) break;
+
+					entry = entry->next;
+				}
+
+				if (!gtk_widget_is_sensitive(dialog_data->extract_full) ||
+				    !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->extract_full)) ||
+				    (i != 1) || !archive->root_entry->child->is_dir)
+				{
+					gchar *extraction_dir;
+
+					extraction_dir = xa_create_containing_directory(archive, destination_path);
+
+					if (!extraction_dir)
+					{
+						xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't create directory!"), "");
+						g_free(destination_path);
+						break;
+					}
+
+					g_free(archive->extraction_dir);
+					archive->extraction_dir = xa_escape_bad_chars(extraction_dir, ESCAPES);
+					g_free(extraction_dir);
+				}
+			}
+
 			g_free (destination_path);
 
 			if (gtk_widget_is_sensitive(dialog_data->overwrite_check))
@@ -559,7 +609,7 @@ void xa_parse_extract_dialog_options (XArchive *archive,Extract_dialog_data *dia
 			{
 				model = gtk_tree_view_get_model(GTK_TREE_VIEW(archive->treeview));
 				string = g_strdup (gtk_entry_get_text(GTK_ENTRY(dialog_data->entry2)));
-				gtk_tree_model_foreach(model,(GtkTreeModelForeachFunc)select_matched_rows,string);
+				gtk_tree_model_foreach(model, select_matched_rows, string);
 				selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(archive->treeview));
 				gtk_tree_selection_selected_foreach(selection,(GtkTreeSelectionForeachFunc)xa_concat_selected_filenames,&names);
 				g_free(string);
@@ -573,7 +623,7 @@ void xa_parse_extract_dialog_options (XArchive *archive,Extract_dialog_data *dia
 			}
 
 			archive->status = XARCHIVESTATUS_EXTRACT;
-			(*archive->extract)(archive,names);
+			(*archive->archiver->extract)(archive,names);
 		}
 	}
 	gtk_widget_hide (dialog_data->dialog1);
@@ -638,7 +688,7 @@ Multi_extract_data *xa_create_multi_extract_dialog()
 	add_button = gtk_button_new_from_stock ("gtk-add");
 	gtk_box_pack_end (GTK_BOX (hbox2),add_button,FALSE,FALSE,0);
 	gtk_button_set_focus_on_click (GTK_BUTTON (add_button),FALSE);
-	g_signal_connect ( (gpointer)add_button,"clicked",G_CALLBACK (xa_multi_extract_dialog_select_files_to_add),dialog_data);
+	g_signal_connect(add_button, "clicked", G_CALLBACK(xa_multi_extract_dialog_select_files_to_add), dialog_data);
 
 	remove_button = gtk_button_new_from_stock ("gtk-remove");
 	gtk_widget_set_sensitive (remove_button,FALSE);
@@ -729,16 +779,16 @@ void xa_multi_extract_dialog_add_file (gchar *file_path, Multi_extract_data *dia
 	if (xa.type == XARCHIVETYPE_UNKNOWN || xa.type == XARCHIVETYPE_NOT_FOUND)
 		return;
 
-	if (!list[xa.type])
+	if (!archiver[xa.type].list)
 	{
 		xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Sorry, this archive format is not supported:"), _("The proper archiver is not installed!"));
 		return;
 	}
 
 	archive.type = xa.type;
-	archive.version = xa.version;
+	archive.tag = xa.tag;
 
-	(*ask[xa.type])(&archive);
+	(*archiver[xa.type].ask)(&archive);
 
 	if (!archive.can_extract)
 	{
@@ -753,7 +803,7 @@ void xa_multi_extract_dialog_add_file (gchar *file_path, Multi_extract_data *dia
 	file = g_path_get_basename(file_path);
 	file_utf8 = g_filename_display_name(file);
 	gtk_list_store_append(dialog->files_liststore,&iter);
-	gtk_list_store_set(dialog->files_liststore, &iter, 0, file_utf8, 1, file_size, 2, path_utf8, 3, xa.type, 4, xa.version, -1);
+	gtk_list_store_set(dialog->files_liststore, &iter, 0, file_utf8, 1, file_size, 2, path_utf8, 3, xa.type, 4, xa.tag, -1);
 	dialog->nr++;
 	g_free(file_utf8);
 	g_free(file);
