@@ -191,20 +191,18 @@ static void xa_create_popup_menu ()
 	gtk_widget_show (image11);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (rrename),image11);
 
-	g_signal_connect ((gpointer) open_popupmenu,"activate",	G_CALLBACK(xa_open_with_from_popupmenu),NULL);
-	g_signal_connect ((gpointer) view,	"activate",	G_CALLBACK(xa_view_from_popupmenu),NULL);
-	g_signal_connect ((gpointer) cut,	"activate",	G_CALLBACK(xa_clipboard_cut),NULL);
-	g_signal_connect ((gpointer) copy,	"activate",	G_CALLBACK(xa_clipboard_copy),NULL);
-	g_signal_connect ((gpointer) paste,	"activate",	G_CALLBACK(xa_clipboard_paste),NULL);
-	g_signal_connect((gpointer) eextract, "activate", G_CALLBACK(xa_extract_archive), NULL);
-	g_signal_connect ((gpointer) ddelete,"activate",G_CALLBACK(xa_delete_archive),NULL);
-	g_signal_connect ((gpointer) rrename,"activate",G_CALLBACK(xa_rename_archive),NULL);
+	g_signal_connect(open_popupmenu, "activate", G_CALLBACK(xa_open_with_from_popupmenu), NULL);
+	g_signal_connect(view, "activate", G_CALLBACK(xa_view_from_popupmenu), NULL);
+	g_signal_connect(cut, "activate", G_CALLBACK(xa_clipboard_cut), NULL);
+	g_signal_connect(copy, "activate", G_CALLBACK(xa_clipboard_copy), NULL);
+	g_signal_connect(paste, "activate", G_CALLBACK(xa_clipboard_paste), NULL);
+	g_signal_connect(eextract, "activate", G_CALLBACK(xa_extract_archive), NULL);
+	g_signal_connect(ddelete, "activate", G_CALLBACK(xa_delete_archive), NULL);
+	g_signal_connect(rrename, "activate", G_CALLBACK(xa_rename_archive), NULL);
 }
 
-static void xa_dir_sidebar_row_expanded (GtkTreeView *tree_view, GtkTreeIter *iter, GtkTreePath *path, gpointer data)
+static void xa_dir_sidebar_row_expanded (GtkTreeView *tree_view, GtkTreeIter *iter, GtkTreePath *path, gpointer model)
 {
-	GtkTreeModel *model = data;
-
 	if (gtk_tree_view_row_expanded(tree_view,path))
 		gtk_tree_store_set(GTK_TREE_STORE(model),iter,0,"gtk-open",-1);
 	else
@@ -217,7 +215,6 @@ static void xa_dir_sidebar_drag_data_received (GtkWidget *widget, GdkDragContext
 	gchar *filename = NULL;
 	gchar *name = NULL;
 	unsigned int len = 0;
-	gint current_page;
 	gint idx;
 	GSList *list = NULL;
 	GtkTreeModel *model;
@@ -228,8 +225,9 @@ static void xa_dir_sidebar_drag_data_received (GtkWidget *widget, GdkDragContext
 	gboolean full_path, dummy_password;
 
 	model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
-	current_page = gtk_notebook_get_current_page(notebook);
-	idx = xa_find_archive_index(current_page);
+
+	idx = xa_find_archive_index(gtk_notebook_get_current_page(notebook));
+
 	if (idx < 0)
 	{
 		gtk_drag_finish(context,FALSE,FALSE,time);
@@ -303,7 +301,7 @@ static void xa_dir_sidebar_drag_data_received (GtkWidget *widget, GdkDragContext
 	gtk_drag_finish (context,TRUE,FALSE,time);
 }
 
-static gboolean xa_dir_sidebar_drag_motion_expand_timeout (gpointer data)
+static gboolean xa_dir_sidebar_drag_motion_expand_timeout (gpointer user_data)
 {
 	GtkTreePath *path;
 
@@ -326,7 +324,7 @@ static gboolean xa_dir_sidebar_drag_motion (GtkWidget *widget, GdkDragContext *c
 	gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (widget),x,y,&path,NULL);
 	if (path)
 	{
-		g_timeout_add_full(G_PRIORITY_LOW, 1000, (GSourceFunc) xa_dir_sidebar_drag_motion_expand_timeout, NULL, NULL);
+		g_timeout_add_full(G_PRIORITY_LOW, 1000, xa_dir_sidebar_drag_motion_expand_timeout, NULL, NULL);
 		g_object_set_data(G_OBJECT(context),"current_path",path);
 	}
 	/* This to set the focus on the dropped row */
@@ -430,11 +428,9 @@ static void xa_select_by_pattern_dialog (GtkMenuItem *menuitem, gpointer user_da
 	GtkWidget *alignment2;
 	gchar *string;
 	gboolean done = FALSE;
-	gint current_page;
-	gint id;
+	gint idx;
 
-	current_page = gtk_notebook_get_current_page (notebook);
-	id = xa_find_archive_index (current_page);
+	idx = xa_find_archive_index(gtk_notebook_get_current_page(notebook));
 
 	ddialog1 = gtk_dialog_new ();
 	gtk_window_set_title (GTK_WINDOW (ddialog1),_("Select by Pattern"));
@@ -507,25 +503,23 @@ static void xa_select_by_pattern_dialog (GtkMenuItem *menuitem, gpointer user_da
 			break;
 		}
 	}
-	gtk_tree_model_foreach(archive[id]->model,(GtkTreeModelForeachFunc)select_matched_rows,string);
+	gtk_tree_model_foreach(archive[idx]->model, select_matched_rows, string);
 	g_free(string);
 
 destroy_delete_dialog:
 	gtk_widget_destroy (ddialog1);
 }
 
-static void xa_handle_navigation_buttons (GtkMenuItem *menuitem, gpointer user_data)
+static void xa_handle_navigation_buttons (GtkToolButton *button, gpointer data)
 {
-	unsigned short int bp = GPOINTER_TO_UINT(user_data);
-	gint current_page;
+	unsigned short int bp = GPOINTER_TO_UINT(data);
 	gint idx;
 	XEntry *new_entry = NULL;
 	GtkTreeIter iter;
 	GtkTreeSelection *selection;
 	GtkTreeModel *model;
 
-	current_page = gtk_notebook_get_current_page (notebook);
-	idx = xa_find_archive_index (current_page);
+	idx = xa_find_archive_index(gtk_notebook_get_current_page(notebook));
 
 	switch (bp)
 	{
@@ -596,11 +590,6 @@ static void xa_handle_navigation_buttons (GtkMenuItem *menuitem, gpointer user_d
 	}
 }
 
-static void xa_close_page (GtkWidget *widget, gpointer data)
-{
-	xa_close_archive (NULL,data);
-}
-
 static void set_label (GtkWidget *label, gchar *text)
 {
     gchar *tmp_markup = g_strdup_printf ("<b>%s</b>",text);
@@ -1081,7 +1070,7 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	gtk_notebook_set_scrollable (notebook,TRUE);
 	gtk_notebook_popup_enable (notebook);
 	gtk_widget_show (GTK_WIDGET(notebook));
-	g_signal_connect ((gpointer) notebook,"switch-page",G_CALLBACK (xa_page_has_changed),NULL);
+	g_signal_connect(notebook, "switch-page", G_CALLBACK(xa_page_has_changed), NULL);
 
 	gtk_drag_dest_set (GTK_WIDGET(notebook),GTK_DEST_DEFAULT_ALL,drop_targets,1,GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
 	g_signal_connect (G_OBJECT (notebook),"drag-data-received",G_CALLBACK (on_drag_data_received),NULL);
@@ -1121,43 +1110,43 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	gtk_box_pack_start (GTK_BOX (hbox_sb),red_led,FALSE,FALSE,0);
 	gtk_misc_set_alignment (GTK_MISC (red_led),1,1);
 
-	g_signal_connect ((gpointer) new1,"activate",G_CALLBACK (xa_new_archive),NULL);
-	g_signal_connect ((gpointer) open1,"activate",G_CALLBACK (xa_open_archive),NULL);
-	g_signal_connect ((gpointer) listing_text,"activate",G_CALLBACK (xa_list_archive),GUINT_TO_POINTER(0));
-	g_signal_connect ((gpointer) listing_html,"activate",G_CALLBACK (xa_list_archive),GUINT_TO_POINTER(1));
-	g_signal_connect ((gpointer) save1,"activate",G_CALLBACK (xa_save_archive),NULL);
-	g_signal_connect ((gpointer) check_menu,"activate",G_CALLBACK (xa_test_archive),NULL);
-	g_signal_connect ((gpointer) properties,"activate",G_CALLBACK (xa_archive_properties),NULL);
-	g_signal_connect ((gpointer) extract_menu,"activate",G_CALLBACK (xa_extract_archive),NULL);
-	g_signal_connect ((gpointer) exe_menu,"activate",G_CALLBACK (xa_convert_sfx),NULL);
-	g_signal_connect ((gpointer) addfile,"activate",G_CALLBACK (xa_add_files_archive),NULL);
-	g_signal_connect ((gpointer) view_shell_output1,"activate",G_CALLBACK (xa_show_archive_output),NULL);
-	g_signal_connect ((gpointer) select_all,"activate",G_CALLBACK (xa_select_all),NULL);
-	g_signal_connect ((gpointer) deselect_all,"activate",G_CALLBACK (xa_deselect_all),NULL);
-	g_signal_connect ((gpointer) select_pattern,"activate",G_CALLBACK (xa_select_by_pattern_dialog),NULL);
-	g_signal_connect ((gpointer) password_entry_menu,"activate",G_CALLBACK (xa_enter_password),NULL);
-	g_signal_connect ((gpointer) prefs_menu,"activate",G_CALLBACK (xa_show_prefs_dialog),NULL);
-	g_signal_connect ((gpointer) close1,"activate",G_CALLBACK (xa_close_archive),NULL);
-	g_signal_connect ((gpointer) quit1,"activate",G_CALLBACK (xa_quit_application),NULL);
-	g_signal_connect ((gpointer) delete_menu,"activate",G_CALLBACK (xa_delete_archive),NULL);
-	g_signal_connect ((gpointer) rename_menu,"activate",G_CALLBACK (xa_rename_archive),NULL);
-	g_signal_connect ((gpointer) comment_menu,"activate",G_CALLBACK (xa_show_archive_comment),NULL);
-	g_signal_connect ((gpointer) multi_extract_menu,"activate",G_CALLBACK (xa_show_multi_extract_dialog),NULL);
-	g_signal_connect ((gpointer) help1,"activate",G_CALLBACK (xa_show_help),NULL);
-	g_signal_connect ((gpointer) about1,"activate",G_CALLBACK (xa_about),NULL);
-	g_signal_connect ((gpointer) unsort_menu,"activate",G_CALLBACK (xa_unsort),NULL);
+	g_signal_connect(new1, "activate", G_CALLBACK(xa_new_archive), NULL);
+	g_signal_connect(open1, "activate", G_CALLBACK(xa_open_archive), NULL);
+	g_signal_connect(listing_text, "activate", G_CALLBACK(xa_list_archive), GUINT_TO_POINTER(0));
+	g_signal_connect(listing_html, "activate", G_CALLBACK(xa_list_archive), GUINT_TO_POINTER(1));
+	g_signal_connect(save1, "activate", G_CALLBACK(xa_save_archive), NULL);
+	g_signal_connect(check_menu, "activate", G_CALLBACK(xa_test_archive), NULL);
+	g_signal_connect(properties, "activate", G_CALLBACK(xa_archive_properties), NULL);
+	g_signal_connect(extract_menu, "activate", G_CALLBACK(xa_extract_archive), NULL);
+	g_signal_connect(exe_menu, "activate", G_CALLBACK(xa_convert_sfx), NULL);
+	g_signal_connect(addfile, "activate", G_CALLBACK(xa_add_files_archive), NULL);
+	g_signal_connect(view_shell_output1, "activate", G_CALLBACK(xa_show_archive_output), NULL);
+	g_signal_connect(select_all, "activate", G_CALLBACK(xa_select_all), NULL);
+	g_signal_connect(deselect_all, "activate", G_CALLBACK(xa_deselect_all), NULL);
+	g_signal_connect(select_pattern, "activate", G_CALLBACK(xa_select_by_pattern_dialog), NULL);
+	g_signal_connect(password_entry_menu, "activate", G_CALLBACK(xa_enter_password), NULL);
+	g_signal_connect(prefs_menu, "activate", G_CALLBACK(xa_show_prefs_dialog), NULL);
+	g_signal_connect(close1, "activate", G_CALLBACK(xa_close_archive), NULL);
+	g_signal_connect(quit1, "activate", G_CALLBACK(xa_quit_application), NULL);
+	g_signal_connect(delete_menu, "activate", G_CALLBACK(xa_delete_archive), NULL);
+	g_signal_connect(rename_menu, "activate", G_CALLBACK(xa_rename_archive), NULL);
+	g_signal_connect(comment_menu, "activate", G_CALLBACK(xa_show_archive_comment), NULL);
+	g_signal_connect(multi_extract_menu, "activate", G_CALLBACK(xa_show_multi_extract_dialog), NULL);
+	g_signal_connect(help1, "activate", G_CALLBACK(xa_show_help), NULL);
+	g_signal_connect(about1, "activate", G_CALLBACK(xa_about), NULL);
+	g_signal_connect(unsort_menu, "activate", G_CALLBACK(xa_unsort), NULL);
 
-	g_signal_connect ((gpointer) New_button,	"clicked",G_CALLBACK (xa_new_archive),NULL);
-	g_signal_connect ((gpointer) Open_button,	"clicked",G_CALLBACK (xa_open_archive),NULL);
+	g_signal_connect(New_button, "clicked", G_CALLBACK(xa_new_archive), NULL);
+	g_signal_connect(Open_button, "clicked", G_CALLBACK(xa_open_archive), NULL);
 
-	g_signal_connect ((gpointer) back_button,	"clicked",G_CALLBACK (xa_handle_navigation_buttons),GUINT_TO_POINTER(1));
-	g_signal_connect ((gpointer) up_button,		"clicked",G_CALLBACK (xa_handle_navigation_buttons),GUINT_TO_POINTER(2));
-	g_signal_connect ((gpointer) forward_button,"clicked",G_CALLBACK (xa_handle_navigation_buttons),GUINT_TO_POINTER(3));
-	g_signal_connect ((gpointer) home_button,	"clicked",G_CALLBACK (xa_handle_navigation_buttons),GUINT_TO_POINTER(0));
+	g_signal_connect(back_button, "clicked", G_CALLBACK(xa_handle_navigation_buttons), GUINT_TO_POINTER(1));
+	g_signal_connect(up_button, "clicked", G_CALLBACK(xa_handle_navigation_buttons), GUINT_TO_POINTER(2));
+	g_signal_connect(forward_button, "clicked", G_CALLBACK(xa_handle_navigation_buttons), GUINT_TO_POINTER(3));
+	g_signal_connect(home_button, "clicked", G_CALLBACK(xa_handle_navigation_buttons), GUINT_TO_POINTER(0));
 
-	g_signal_connect ((gpointer) AddFile_button,"clicked",G_CALLBACK (xa_add_files_archive),NULL);
-    g_signal_connect ((gpointer) Extract_button,"clicked",G_CALLBACK (xa_extract_archive),	NULL);
-	g_signal_connect ((gpointer) Stop_button,	"clicked",G_CALLBACK (xa_cancel_archive),	NULL);
+	g_signal_connect(AddFile_button, "clicked", G_CALLBACK(xa_add_files_archive), NULL);
+	g_signal_connect(Extract_button, "clicked", G_CALLBACK(xa_extract_archive), NULL);
+	g_signal_connect(Stop_button, "clicked", G_CALLBACK(xa_cancel_archive), NULL);
 	gtk_window_add_accel_group (GTK_WINDOW (xa_main_window),accel_group);
 }
 
@@ -1225,7 +1214,7 @@ void xa_add_page (XArchive *archive)
 	gtk_button_set_focus_on_click(GTK_BUTTON(close_button),FALSE);
 	gtk_button_set_relief (GTK_BUTTON(close_button),GTK_RELIEF_NONE);
 	gtk_widget_set_tooltip_text(close_button, _("Close archive"));
-	g_signal_connect(G_OBJECT(close_button), "clicked", G_CALLBACK(xa_close_page), archive->page);
+	g_signal_connect(G_OBJECT(close_button), "clicked", G_CALLBACK(xa_close_archive), archive->page);
 
 	rcstyle = gtk_rc_style_new();
 	rcstyle->xthickness = rcstyle->ythickness = 0;
@@ -1250,13 +1239,13 @@ void xa_add_page (XArchive *archive)
 	gtk_container_add(GTK_CONTAINER(archive->page), archive->treeview);
 	gtk_widget_show (archive->treeview);
 	gtk_tree_view_set_rules_hint ( GTK_TREE_VIEW (archive->treeview),TRUE);
-	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (archive->treeview),(GtkTreeViewSearchEqualFunc) treeview_select_search,NULL,NULL);
+	gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(archive->treeview), treeview_select_search, NULL, NULL);
 	GtkTreeSelection *sel = gtk_tree_view_get_selection( GTK_TREE_VIEW (archive->treeview));
 	gtk_tree_selection_set_mode(sel,GTK_SELECTION_MULTIPLE);
 	gtk_tree_view_set_rubber_banding(GTK_TREE_VIEW(archive->treeview),TRUE);
 
 	gtk_drag_source_set (archive->treeview,GDK_BUTTON1_MASK,drag_targets,1,GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
-	g_signal_connect ((gpointer) sel,			   "changed",		G_CALLBACK (xa_row_selected),archive);
+	g_signal_connect(sel, "changed", G_CALLBACK(xa_row_selected), archive);
 	g_signal_connect (G_OBJECT (archive->treeview),"drag-begin",	G_CALLBACK (drag_begin),archive);
 	g_signal_connect (G_OBJECT (archive->treeview),"drag-data-get", G_CALLBACK (drag_data_get),archive);
 	g_signal_connect (G_OBJECT (archive->treeview),"drag-end",		G_CALLBACK (drag_end),NULL);
@@ -1365,16 +1354,13 @@ gboolean xa_check_password (XArchive *archive)
 	return (archive->password != NULL);
 }
 
-gboolean select_matched_rows(GtkTreeModel *model,GtkTreePath *path,GtkTreeIter *iter,gpointer data)
+gboolean select_matched_rows (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer string)
 {
-	gchar *string = data;
 	char **patterns;
 	XEntry *entry = NULL;
-	gint current_page;
 	gint idx;
 
-	current_page = gtk_notebook_get_current_page(notebook);
-	idx = xa_find_archive_index (current_page);
+	idx = xa_find_archive_index(gtk_notebook_get_current_page(notebook));
 	patterns = g_strsplit(string,";",-1);
 
 	gtk_tree_model_get(model, iter, archive[idx]->columns - 1, &entry, -1);
@@ -1678,7 +1664,7 @@ void xa_show_progress_bar (XArchive *archive)
 		gtk_box_pack_end (GTK_BOX (action_area),cancel_button,TRUE,TRUE,12);
 
 		g_signal_connect(G_OBJECT(cancel_button), "clicked", G_CALLBACK(xa_cancel_progress_bar), &archive->child_pid);
-		g_signal_connect(G_OBJECT(progress->window), "delete_event", G_CALLBACK(xa_close_progress_bar), &archive->child_pid);
+		g_signal_connect(G_OBJECT(progress->window), "delete-event", G_CALLBACK(xa_close_progress_bar), &archive->child_pid);
 	}
 	gtk_widget_show_all(progress->window);
 }
@@ -1699,7 +1685,7 @@ void xa_increase_progress_bar (Progress *progress, gchar *filename, double perce
 		g_free (markup);
 }
 
-gboolean xa_pulse_progress_bar (gpointer data)
+gboolean xa_pulse_progress_bar (gpointer user_data)
 {
 	if (progress && gtk_widget_get_visible(progress->window))
 	{
