@@ -20,19 +20,15 @@
 #ifndef LHA_H
 #define LHA_H
 
-#include <gtk/gtk.h>
-#include <string.h>
-#include <stdlib.h>
-#include "support.h"
-#include "string_utils.h"
+#include <glib.h>
 #include "archive.h"
 
-void xa_lha_delete (XArchive *,GSList *);
-void xa_lha_add (XArchive *,GString *,gchar *);
-gboolean xa_lha_extract(XArchive *,GSList *);
-void xa_lha_test (XArchive *);
-void xa_get_lha_line_content (gchar *line, gpointer data);
-void xa_open_lha (XArchive *archive);
-gboolean jump_header, last_line;
-gboolean isLha (FILE *ptr);
+void xa_lha_add(XArchive *, GSList *, gchar *);
+void xa_lha_ask(XArchive *);
+gboolean xa_lha_check_program(gchar *);
+void xa_lha_delete(XArchive *, GSList *);
+gboolean xa_lha_extract(XArchive *, GSList *);
+void xa_lha_list(XArchive *);
+void xa_lha_test(XArchive *);
+
 #endif
