@@ -49,13 +49,6 @@ GtkWidget *xa_main_window;
 
 XArchiver archiver[XARCHIVETYPE_TYPES];
 
-ask_func ask[XARCHIVETYPE_TYPES];
-list_func list[XARCHIVETYPE_TYPES];
-test_func test[XARCHIVETYPE_TYPES];
-extract_func extract[XARCHIVETYPE_TYPES];
-add_func add[XARCHIVETYPE_TYPES];
-delete_func delete[XARCHIVETYPE_TYPES];
-
 gchar *xdg_open;
 
 Add_dialog_data *add_window;
@@ -63,7 +56,7 @@ Extract_dialog_data *extract_window;
 Multi_extract_data *multi_extract_window;
 Prefs_dialog_data *prefs_window;
 
-static gchar *opt_extract_path, *opt_compress;
+static gchar *opt_extract_path, *opt_ensure_dir, *opt_compress;
 static gboolean opt_extract, opt_multi_extract, opt_add, opt_info, opt_version;
 
 static GOptionEntry entries[] =
@@ -76,6 +69,10 @@ static GOptionEntry entries[] =
 		N_("Extract archive by asking the extraction\n                                     directory and quit"),
 		NULL
 	},
+	{	"ensure-directory", 'd', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_ensure_dir,
+		N_("Extract archive to a containing directory\n                                     and quit"),
+		NULL
+	},
 	{	"multi-extract", 'm', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_multi_extract,
 		N_("Extract multiple archives by asking the\n                                     extraction directory and quit"),
 		NULL
@@ -100,7 +97,7 @@ static GOptionEntry entries[] =
 static void xa_check_available_archivers ()
 {
 	XArchiveType type;
-	gchar *path, *cpio, *lsar, *sevenz, *unar, *xz;
+	gchar *path, *cpio, *lsar, *sevenz, *unar, *xz, *zstd;
 	gboolean is7z = TRUE, is7za = TRUE, is7zr = TRUE, standard;
 
 	/* (un)compressors that can handle various types */
@@ -126,6 +123,7 @@ static void xa_check_available_archivers ()
 	lsar = g_find_program_in_path("lsar");
 	unar = g_find_program_in_path("unar");
 	xz = g_find_program_in_path("xz");
+	zstd = xa_gzip_et_al_check_zstd("zstd", "unzstd", &archiver[XARCHIVETYPE_ZSTD].is_compressor);
 
 	/* 7-zip */
 
@@ -151,12 +149,12 @@ static void xa_check_available_archivers ()
 		archiver[type].type = g_slist_append(archiver[type].type, "7zip");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.7z");
 
-		ask[type] = (standard ? xa_7zip_ask : xa_unar_ask);
-		list[type] = (standard ? xa_7zip_list : xa_unar_list);
-		test[type] = (standard ? xa_7zip_test : xa_unar_test);
-		extract[type] = (standard ? xa_7zip_extract : (unar ? xa_unar_extract : NULL));
-		add[type] = (standard ? xa_7zip_add : NULL);
-		delete[type] = (standard ? xa_7zip_delete : NULL);
+		archiver[type].ask = (standard ? xa_7zip_ask : xa_unar_ask);
+		archiver[type].list = (standard ? xa_7zip_list : xa_unar_list);
+		archiver[type].test = (standard ? xa_7zip_test : xa_unar_test);
+		archiver[type].extract = (standard ? xa_7zip_extract : (unar ? xa_unar_extract : NULL));
+		archiver[type].add = (standard ? xa_7zip_add : NULL);
+		archiver[type].delete = (standard ? xa_7zip_delete : NULL);
 	}
 
 	/* ARJ */
@@ -193,12 +191,12 @@ static void xa_check_available_archivers ()
 		archiver[type].type = g_slist_append(archiver[type].type, "arj");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.arj");
 
-		ask[type] = FUNC(standard, xa_arj_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
-		list[type] = FUNC(standard, xa_arj_list, is7z, xa_7zip_list, lsar, xa_unar_list);
-		test[type] = FUNC(standard, xa_arj_test, is7z, xa_7zip_test, lsar, xa_unar_test);
-		extract[type] = FUNC(standard, xa_arj_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
-		add[type] = FUNC(standard, xa_arj_add, is7z, NULL, lsar, NULL);
-		delete[type] = FUNC(standard, xa_arj_delete, is7z, NULL, lsar, NULL);
+		archiver[type].ask = FUNC(standard, xa_arj_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
+		archiver[type].list = FUNC(standard, xa_arj_list, is7z, xa_7zip_list, lsar, xa_unar_list);
+		archiver[type].test = FUNC(standard, xa_arj_test, is7z, xa_7zip_test, lsar, xa_unar_test);
+		archiver[type].extract = FUNC(standard, xa_arj_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
+		archiver[type].add = FUNC(standard, xa_arj_add, is7z, NULL, lsar, NULL);
+		archiver[type].delete = FUNC(standard, xa_arj_delete, is7z, NULL, lsar, NULL);
 	}
 
 	/* bzip2 */
@@ -241,11 +239,11 @@ static void xa_check_available_archivers ()
 		archiver[type].type = g_slist_append(archiver[type].type, "bzip2");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.bz2");
 
-		ask[type] = FUNC(standard, xa_gzip_et_al_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
-		list[type] = FUNC(standard, xa_gzip_et_al_list, is7za, xa_7zip_list, lsar, xa_unar_list);
-		test[type] = FUNC(standard, xa_gzip_et_al_test, is7za, xa_7zip_test, lsar, xa_unar_test);
-		extract[type] = FUNC(standard, xa_gzip_et_al_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
-		add[type] = FUNC(standard, xa_gzip_et_al_add, is7za, xa_7zip_add, lsar, NULL);
+		archiver[type].ask = FUNC(standard, xa_gzip_et_al_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
+		archiver[type].list = FUNC(standard, xa_gzip_et_al_list, is7za, xa_7zip_list, lsar, xa_unar_list);
+		archiver[type].test = FUNC(standard, xa_gzip_et_al_test, is7za, xa_7zip_test, lsar, xa_unar_test);
+		archiver[type].extract = FUNC(standard, xa_gzip_et_al_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
+		archiver[type].add = FUNC(standard, xa_gzip_et_al_add, is7za, xa_7zip_add, lsar, NULL);
 	}
 
 	/* compress */
@@ -253,6 +251,11 @@ static void xa_check_available_archivers ()
 	type = XARCHIVETYPE_COMPRESS;
 	path = g_find_program_in_path("compress");
 
+	if (path)
+		archiver[type].is_compressor = TRUE;
+	else
+		path = g_find_program_in_path("uncompress");
+
 	standard = (path != NULL);
 
 	if (!standard)
@@ -274,15 +277,14 @@ static void xa_check_available_archivers ()
 	if (path)
 	{
 		archiver[type].program[0] = path;
-		archiver[type].is_compressor = standard;
 		archiver[type].type = g_slist_append(archiver[type].type, "compress");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.Z");
 
-		ask[type] = FUNC(standard, xa_gzip_et_al_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
-		list[type] = FUNC(standard, xa_gzip_et_al_list, is7za, xa_7zip_list, lsar, xa_unar_list);
-		test[type] = FUNC(standard, xa_gzip_et_al_test, is7za, xa_7zip_test, lsar, NULL);
-		extract[type] = FUNC(standard, xa_gzip_et_al_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
-		add[type] = FUNC(standard, xa_gzip_et_al_add, is7za, NULL, lsar, NULL);
+		archiver[type].ask = FUNC(standard, xa_gzip_et_al_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
+		archiver[type].list = FUNC(standard, xa_gzip_et_al_list, is7za, xa_7zip_list, lsar, xa_unar_list);
+		archiver[type].test = FUNC(standard, xa_gzip_et_al_test, is7za, xa_7zip_test, lsar, NULL);
+		archiver[type].extract = FUNC(standard, xa_gzip_et_al_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
+		archiver[type].add = FUNC(standard, xa_gzip_et_al_add, is7za, NULL, lsar, NULL);
 	}
 
 	/* cpio */
@@ -315,11 +317,11 @@ static void xa_check_available_archivers ()
 		archiver[type].type = g_slist_append(archiver[type].type, "cpio");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.cpio");
 
-		ask[type] = FUNC(standard, xa_cpio_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
-		list[type] = FUNC(standard, xa_cpio_list, is7z, xa_7zip_list, lsar, xa_unar_list);
-		test[type] = FUNC(standard, NULL, is7z, xa_7zip_test, lsar, NULL);
-		extract[type] = FUNC(standard, xa_cpio_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
-		add[type] = FUNC(standard, xa_cpio_add, is7z, NULL, lsar, NULL);
+		archiver[type].ask = FUNC(standard, xa_cpio_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
+		archiver[type].list = FUNC(standard, xa_cpio_list, is7z, xa_7zip_list, lsar, xa_unar_list);
+		archiver[type].test = FUNC(standard, NULL, is7z, xa_7zip_test, lsar, NULL);
+		archiver[type].extract = FUNC(standard, xa_cpio_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
+		archiver[type].add = FUNC(standard, xa_cpio_add, is7z, NULL, lsar, NULL);
 	}
 
 	/* debian package */
@@ -349,9 +351,9 @@ static void xa_check_available_archivers ()
 		archiver[type].type = g_slist_append(archiver[type].type, "deb");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.deb");
 
-		ask[type] = FUNC(standard, xa_deb_ask, lsar && unar, xa_unar_ask, is7z, xa_7zip_ask);
-		list[type] = FUNC(standard, xa_deb_list, lsar && unar, xa_unar_list, is7z, xa_7zip_list);
-		extract[type]  = FUNC(standard, xa_deb_extract, lsar && unar, xa_unar_extract, is7z, xa_7zip_extract);
+		archiver[type].ask = FUNC(standard, xa_deb_ask, lsar && unar, xa_unar_ask, is7z, xa_7zip_ask);
+		archiver[type].list = FUNC(standard, xa_deb_list, lsar && unar, xa_unar_list, is7z, xa_7zip_list);
+		archiver[type].extract  = FUNC(standard, xa_deb_extract, lsar && unar, xa_unar_extract, is7z, xa_7zip_extract);
 	}
 
 	/* GNU zip */
@@ -394,11 +396,11 @@ static void xa_check_available_archivers ()
 		archiver[type].type = g_slist_append(archiver[type].type, "gzip");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.gz");
 
-		ask[type] = FUNC(standard, xa_gzip_et_al_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
-		list[type] = FUNC(standard, xa_gzip_et_al_list, is7za, xa_7zip_list, lsar, xa_unar_list);
-		test[type] = FUNC(standard, xa_gzip_et_al_test, is7za, xa_7zip_test, lsar, xa_unar_test);
-		extract[type] = FUNC(standard, xa_gzip_et_al_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
-		add[type] = FUNC(standard, xa_gzip_et_al_add, is7za, xa_7zip_add, lsar, NULL);
+		archiver[type].ask = FUNC(standard, xa_gzip_et_al_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
+		archiver[type].list = FUNC(standard, xa_gzip_et_al_list, is7za, xa_7zip_list, lsar, xa_unar_list);
+		archiver[type].test = FUNC(standard, xa_gzip_et_al_test, is7za, xa_7zip_test, lsar, xa_unar_test);
+		archiver[type].extract = FUNC(standard, xa_gzip_et_al_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
+		archiver[type].add = FUNC(standard, xa_gzip_et_al_add, is7za, xa_7zip_add, lsar, NULL);
 	}
 
 	/* LHA */
@@ -431,12 +433,37 @@ static void xa_check_available_archivers ()
 		archiver[type].type = g_slist_append(archiver[type].type, "lha");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lzh");
 
-		ask[type] = FUNC(standard, xa_lha_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
-		list[type] = FUNC(standard, xa_lha_list, is7z, xa_7zip_list, lsar, xa_unar_list);
-		test[type] = FUNC(standard, xa_lha_test, is7z, xa_7zip_test, lsar, xa_unar_test);
-		extract[type] = FUNC(standard, xa_lha_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
-		add[type] = FUNC(standard, xa_lha_add, is7z, NULL, lsar, NULL);
-		delete[type] = FUNC(standard, xa_lha_delete, is7z, NULL, lsar, NULL);
+		archiver[type].ask = FUNC(standard, xa_lha_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
+		archiver[type].list = FUNC(standard, xa_lha_list, is7z, xa_7zip_list, lsar, xa_unar_list);
+		archiver[type].test = FUNC(standard, xa_lha_test, is7z, xa_7zip_test, lsar, xa_unar_test);
+		archiver[type].extract = FUNC(standard, xa_lha_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
+		archiver[type].add = FUNC(standard, xa_lha_add, is7z, NULL, lsar, NULL);
+		archiver[type].delete = FUNC(standard, xa_lha_delete, is7z, NULL, lsar, NULL);
+	}
+
+	/* lrzip */
+
+	type = XARCHIVETYPE_LRZIP;
+	path = g_find_program_in_path("lrzip");
+
+	if (path)
+		archiver[type].is_compressor = TRUE;
+	else
+		path = g_find_program_in_path("lrunzip");
+
+	if (path)
+	{
+		xa_gzip_et_al_check_lrzip(path);
+
+		archiver[type].program[0] = path;
+		archiver[type].type = g_slist_append(archiver[type].type, "lrzip");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lrz");
+
+		archiver[type].ask = xa_gzip_et_al_ask;
+		archiver[type].list = xa_gzip_et_al_list;
+		archiver[type].test = xa_gzip_et_al_test;
+		archiver[type].extract = xa_gzip_et_al_extract;
+		archiver[type].add = xa_gzip_et_al_add;
 	}
 
 	/* lz4 */
@@ -444,18 +471,22 @@ static void xa_check_available_archivers ()
 	type = XARCHIVETYPE_LZ4;
 	path = g_find_program_in_path("lz4");
 
+	if (path)
+		archiver[type].is_compressor = TRUE;
+	else
+		path = g_find_program_in_path("unlz4");
+
 	if (path)
 	{
 		archiver[type].program[0] = path;
-		archiver[type].is_compressor = TRUE;
 		archiver[type].type = g_slist_append(archiver[type].type, "lz4");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lz4");
 
-		ask[type] = xa_gzip_et_al_ask;
-		list[type] = xa_gzip_et_al_list;
-		test[type] = xa_gzip_et_al_test;
-		extract[type] = xa_gzip_et_al_extract;
-		add[type] = xa_gzip_et_al_add;
+		archiver[type].ask = xa_gzip_et_al_ask;
+		archiver[type].list = xa_gzip_et_al_list;
+		archiver[type].test = xa_gzip_et_al_test;
+		archiver[type].extract = xa_gzip_et_al_extract;
+		archiver[type].add = xa_gzip_et_al_add;
 	}
 
 	/* lzip */
@@ -470,11 +501,11 @@ static void xa_check_available_archivers ()
 		archiver[type].type = g_slist_append(archiver[type].type, "lzip");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lz");
 
-		ask[type] = xa_gzip_et_al_ask;
-		list[type] = xa_gzip_et_al_list;
-		test[type] = xa_gzip_et_al_test;
-		extract[type] = xa_gzip_et_al_extract;
-		add[type] = xa_gzip_et_al_add;
+		archiver[type].ask = xa_gzip_et_al_ask;
+		archiver[type].list = xa_gzip_et_al_list;
+		archiver[type].test = xa_gzip_et_al_test;
+		archiver[type].extract = xa_gzip_et_al_extract;
+		archiver[type].add = xa_gzip_et_al_add;
 	}
 
 	/* lzma */
@@ -486,6 +517,11 @@ static void xa_check_available_archivers ()
 		/* alternative compressor */
 		path = g_strconcat(xz, " --format=lzma", NULL);
 
+	if (path)
+		archiver[type].is_compressor = TRUE;
+	else
+		path = g_find_program_in_path("unlzma");
+
 	standard = (path != NULL);
 
 	if (!standard)
@@ -507,15 +543,14 @@ static void xa_check_available_archivers ()
 	if (path)
 	{
 		archiver[type].program[0] = path;
-		archiver[type].is_compressor = standard;
 		archiver[type].type = g_slist_append(archiver[type].type, "lzma");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lzma");
 
-		ask[type] = FUNC(standard, xa_gzip_et_al_ask, is7zr, xa_7zip_ask, lsar, xa_unar_ask);
-		list[type] = FUNC(standard, xa_gzip_et_al_list, is7zr, xa_7zip_list, lsar, xa_unar_list);
-		test[type] = FUNC(standard, xa_gzip_et_al_test, is7zr, xa_7zip_test, lsar, NULL);
-		extract[type] = FUNC(standard, xa_gzip_et_al_extract, is7zr, xa_7zip_extract, unar, xa_unar_extract);
-		add[type] = FUNC(standard, xa_gzip_et_al_add, is7zr, NULL, lsar, NULL);
+		archiver[type].ask = FUNC(standard, xa_gzip_et_al_ask, is7zr, xa_7zip_ask, lsar, xa_unar_ask);
+		archiver[type].list = FUNC(standard, xa_gzip_et_al_list, is7zr, xa_7zip_list, lsar, xa_unar_list);
+		archiver[type].test = FUNC(standard, xa_gzip_et_al_test, is7zr, xa_7zip_test, lsar, NULL);
+		archiver[type].extract = FUNC(standard, xa_gzip_et_al_extract, is7zr, xa_7zip_extract, unar, xa_unar_extract);
+		archiver[type].add = FUNC(standard, xa_gzip_et_al_add, is7zr, NULL, lsar, NULL);
 	}
 
 	/* lzop */
@@ -523,18 +558,22 @@ static void xa_check_available_archivers ()
 	type = XARCHIVETYPE_LZOP;
 	path = g_find_program_in_path("lzop");
 
+	if (path)
+		archiver[type].is_compressor = TRUE;
+	else
+		path = g_find_program_in_path("unlzop");
+
 	if (path)
 	{
 		archiver[type].program[0] = path;
-		archiver[type].is_compressor = TRUE;
 		archiver[type].type = g_slist_append(archiver[type].type, "lzop");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lzo");
 
-		ask[type] = xa_gzip_et_al_ask;
-		list[type] = xa_gzip_et_al_list;
-		test[type] = xa_gzip_et_al_test;
-		extract[type] = xa_gzip_et_al_extract;
-		add[type] = xa_gzip_et_al_add;
+		archiver[type].ask = xa_gzip_et_al_ask;
+		archiver[type].list = xa_gzip_et_al_list;
+		archiver[type].test = xa_gzip_et_al_test;
+		archiver[type].extract = xa_gzip_et_al_extract;
+		archiver[type].add = xa_gzip_et_al_add;
 	}
 
 	/* RAR */
@@ -578,16 +617,16 @@ static void xa_check_available_archivers ()
 		{
 			archiver[type].type = g_slist_append(archiver[type].type, "rar5");
 			archiver[type].glob = g_slist_append(archiver[type].glob, " .rar");
-			archiver[type].version = g_slist_append(archiver[type].version, GUINT_TO_POINTER(5));
-			archiver[type].version = g_slist_append(archiver[type].version, g_slist_last(archiver[type].type)->data);
+			archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER(5));
+			archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 		}
 
-		ask[type] = FUNC(standard, xa_rar_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
-		list[type] = FUNC(standard, xa_rar_list, is7z, xa_7zip_list, lsar, xa_unar_list);
-		test[type] = FUNC(standard, xa_rar_test, is7z, xa_7zip_test, lsar, xa_unar_test);
-		extract[type] = FUNC(standard, xa_rar_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
-		add[type] = FUNC(standard, xa_rar_add, is7z, NULL, lsar, NULL);
-		delete[type] = FUNC(standard, xa_rar_delete, is7z, NULL, lsar, NULL);
+		archiver[type].ask = FUNC(standard, xa_rar_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
+		archiver[type].list = FUNC(standard, xa_rar_list, is7z, xa_7zip_list, lsar, xa_unar_list);
+		archiver[type].test = FUNC(standard, xa_rar_test, is7z, xa_7zip_test, lsar, xa_unar_test);
+		archiver[type].extract = FUNC(standard, xa_rar_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
+		archiver[type].add = FUNC(standard, xa_rar_add, is7z, NULL, lsar, NULL);
+		archiver[type].delete = FUNC(standard, xa_rar_delete, is7z, NULL, lsar, NULL);
 	}
 
 	/* RPM package */
@@ -619,10 +658,10 @@ static void xa_check_available_archivers ()
 		archiver[type].type = g_slist_append(archiver[type].type, "rpm");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.rpm");
 
-		ask[type] = FUNC(standard, xa_rpm_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
-		list[type] = FUNC(standard, xa_rpm_list, is7z, xa_7zip_list, lsar, xa_unar_list);
-		test[type] = FUNC(standard, NULL, is7z, xa_7zip_test, lsar, xa_unar_test);
-		extract[type] = FUNC(standard, xa_rpm_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
+		archiver[type].ask = FUNC(standard, xa_rpm_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
+		archiver[type].list = FUNC(standard, xa_rpm_list, is7z, xa_7zip_list, lsar, xa_unar_list);
+		archiver[type].test = FUNC(standard, NULL, is7z, xa_7zip_test, lsar, xa_unar_test);
+		archiver[type].extract = FUNC(standard, xa_rpm_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
 	}
 
 	/* tape archive */
@@ -658,12 +697,12 @@ static void xa_check_available_archivers ()
 		archiver[type].type = g_slist_append(archiver[type].type, "tar");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.tar");
 
-		ask[type] = FUNC(standard, xa_tar_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
-		list[type] = FUNC(standard, xa_tar_list, is7za, xa_7zip_list, lsar, xa_unar_list);
-		test[type] = FUNC(standard, NULL, is7za, xa_7zip_test, lsar, NULL);
-		extract[type] = FUNC(standard, xa_tar_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
-		add[type] = FUNC(standard, xa_tar_add, is7za, xa_7zip_add, lsar, NULL);
-		delete[type] = FUNC(standard, xa_tar_delete, is7za, xa_7zip_delete, lsar, NULL);
+		archiver[type].ask = FUNC(standard, xa_tar_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
+		archiver[type].list = FUNC(standard, xa_tar_list, is7za, xa_7zip_list, lsar, xa_unar_list);
+		archiver[type].test = FUNC(standard, NULL, is7za, xa_7zip_test, lsar, NULL);
+		archiver[type].extract = FUNC(standard, xa_tar_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
+		archiver[type].add = FUNC(standard, xa_tar_add, is7za, xa_7zip_add, lsar, NULL);
+		archiver[type].delete = FUNC(standard, xa_tar_delete, is7za, xa_7zip_delete, lsar, NULL);
 	}
 
 	/* xz */
@@ -671,14 +710,23 @@ static void xa_check_available_archivers ()
 	type = XARCHIVETYPE_XZ;
 	path = xz;
 
-	standard = (path != NULL);
+	if (path)
+		archiver[type].is_compressor = TRUE;
+	else
+		path = g_find_program_in_path("unxz");
+
+	standard = (path && archiver[type].is_compressor);
 
 	if (!standard)
 	{
 		if (is7zr)
+		{
+			g_free(path);
 			/* alternative compressor */
 			path = g_strconcat(sevenz, " -txz", NULL);
-		else
+			archiver[type].is_compressor = TRUE;
+		}
+		else if (!path)
 		{
 			if (lsar)
 				/* alternative ... */
@@ -687,37 +735,45 @@ static void xa_check_available_archivers ()
 				/* ... uncompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
+		else
+			standard = TRUE;
 	}
 
 	if (path)
 	{
 		archiver[type].program[0] = path;
-		archiver[type].is_compressor = (standard || is7zr);
 		archiver[type].type = g_slist_append(archiver[type].type, "xz");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.xz");
 
-		ask[type] = FUNC(standard, xa_gzip_et_al_ask, is7zr, xa_7zip_ask, lsar, xa_unar_ask);
-		list[type] = FUNC(standard, xa_gzip_et_al_list, is7zr, xa_7zip_list, lsar, xa_unar_list);
-		test[type] = FUNC(standard, xa_gzip_et_al_test, is7zr, xa_7zip_test, lsar, xa_unar_test);
-		extract[type] = FUNC(standard, xa_gzip_et_al_extract, is7zr, xa_7zip_extract, unar, xa_unar_extract);
-		add[type] = FUNC(standard, xa_gzip_et_al_add, is7zr, xa_7zip_add, lsar, NULL);
+		archiver[type].ask = FUNC(standard, xa_gzip_et_al_ask, is7zr, xa_7zip_ask, lsar, xa_unar_ask);
+		archiver[type].list = FUNC(standard, xa_gzip_et_al_list, is7zr, xa_7zip_list, lsar, xa_unar_list);
+		archiver[type].test = FUNC(standard, xa_gzip_et_al_test, is7zr, xa_7zip_test, lsar, xa_unar_test);
+		archiver[type].extract = FUNC(standard, xa_gzip_et_al_extract, is7zr, xa_7zip_extract, unar, xa_unar_extract);
+		archiver[type].add = FUNC(standard, xa_gzip_et_al_add, is7zr, xa_7zip_add, lsar, NULL);
 	}
 
 	/* zip */
 
 	type = XARCHIVETYPE_ZIP;
-	path = g_find_program_in_path("unzip");
 
-	if (path)
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->prefer_unzip)) ||
+	    (!is7za && !lsar))
 	{
-		gchar *zip = g_find_program_in_path("zip");
+		path = g_find_program_in_path("unzip");
 
-		if (zip)
+		if (path)
 		{
-			archiver[type].program[1] = zip;
-			archiver[type].is_compressor = TRUE;
+			gchar *zip = g_find_program_in_path("zip");
+
+			if (zip)
+			{
+				archiver[type].program[1] = zip;
+				archiver[type].is_compressor = TRUE;
+			}
 		}
 	}
+	else
+		path = NULL;
 
 	standard = (path && archiver[type].is_compressor);
 
@@ -748,23 +804,56 @@ static void xa_check_available_archivers ()
 		archiver[type].program[0] = path;
 		archiver[type].type = g_slist_append(archiver[type].type, "zip");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.zip");
+		/* electronic publication */
+		archiver[type].type = g_slist_append(archiver[type].type, "epub");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.epub");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('e'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 		/* java archive */
 		archiver[type].type = g_slist_append(archiver[type].type, "jar");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.jar");
-		archiver[type].version = g_slist_append(archiver[type].version, GUINT_TO_POINTER('j'));
-		archiver[type].version = g_slist_append(archiver[type].version, g_slist_last(archiver[type].type)->data);
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('j'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
+		/* OpenOffice.org extension */
+		archiver[type].type = g_slist_append(archiver[type].type, "oxt");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.oxt");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('o'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
+		/* XPInstall */
+		archiver[type].type = g_slist_append(archiver[type].type, "xpi");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.xpi");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('x'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 
-		ask[type] = FUNC(standard, xa_zip_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
-		list[type] = FUNC(standard, xa_zip_list, is7za, xa_7zip_list, lsar, xa_unar_list);
-		test[type] = FUNC(standard, xa_zip_test, is7za, xa_7zip_test, lsar, xa_unar_test);
-		extract[type] = FUNC(standard, xa_zip_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
-		add[type] = FUNC(standard, xa_zip_add, is7za, xa_7zip_add, lsar, NULL);
-		delete[type] = FUNC(standard, xa_zip_delete, is7za, xa_7zip_delete, lsar, NULL);
+		archiver[type].ask = FUNC(standard, xa_zip_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
+		archiver[type].list = FUNC(standard, xa_zip_list, is7za, xa_7zip_list, lsar, xa_unar_list);
+		archiver[type].test = FUNC(standard, xa_zip_test, is7za, xa_7zip_test, lsar, xa_unar_test);
+		archiver[type].extract = FUNC(standard, xa_zip_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
+		archiver[type].add = FUNC(standard, xa_zip_add, is7za, xa_7zip_add, lsar, NULL);
+		archiver[type].delete = FUNC(standard, xa_zip_delete, is7za, xa_7zip_delete, lsar, NULL);
+	}
+
+	/* Zstandard */
+
+	type = XARCHIVETYPE_ZSTD;
+	path = zstd;                // either zstd or unzstd
+
+	if (path)
+	{
+		archiver[type].program[0] = path;
+		archiver[type].type = g_slist_append(archiver[type].type, "zstd");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.zst");
+
+		archiver[type].ask = xa_gzip_et_al_ask;
+		archiver[type].list = xa_gzip_et_al_list;
+		archiver[type].test = xa_gzip_et_al_test;
+		archiver[type].extract = xa_gzip_et_al_extract;
+		archiver[type].add = xa_gzip_et_al_add;
 	}
 
 	/* compressed tar */
 
-	if (ask[XARCHIVETYPE_TAR] == xa_tar_ask || ask[XARCHIVETYPE_TAR] == xa_unar_ask)
+	if (archiver[XARCHIVETYPE_TAR].ask == xa_tar_ask || archiver[XARCHIVETYPE_TAR].ask == xa_unar_ask)
 	{
 		struct
 		{
@@ -774,13 +863,15 @@ static void xa_check_available_archivers ()
 		} compressed_tar_infos[] =
 		{
 			{XARCHIVETYPE_BZIP2, {"tar.bzip2", NULL}, {"*.tar.bz2", "*.tbz2"}},
-			{XARCHIVETYPE_COMPRESS, {"tar.Z", NULL}, {"*.tar.Z", ""}},
+			{XARCHIVETYPE_COMPRESS, {"tar.compress", NULL}, {"*.tar.Z", ""}},
 			{XARCHIVETYPE_GZIP, {"tar.gzip", NULL}, {"*.tar.gz", "*.tgz"}},
+			{XARCHIVETYPE_LRZIP, {"tar.lrzip", NULL}, {"*.tar.lrz", "*.tlrz"}},
 			{XARCHIVETYPE_LZ4, {"tar.lz4", NULL}, {"*.tar.lz4", ""}},
 			{XARCHIVETYPE_LZIP, {"tar.lzip", NULL}, {"*.tar.lz", ""}},
 			{XARCHIVETYPE_LZMA, {"tar.lzma", NULL}, {"*.tar.lzma", "*.tlz"}},
 			{XARCHIVETYPE_LZOP, {"tar.lzop", NULL}, {"*.tar.lzo", "*.tzo"}},
 			{XARCHIVETYPE_XZ, {"tar.xz", NULL}, {"*.tar.xz", "*.txz"}},
+			{XARCHIVETYPE_ZSTD, {"tar.zstd", NULL}, {"*.tar.zst", ""}},
 			{XARCHIVETYPE_UNKNOWN, {NULL, NULL}, {NULL, NULL}}
 		}, *i;
 
@@ -793,8 +884,8 @@ static void xa_check_available_archivers ()
 				if (!xa_get_compressed_tar_type(&type))
 					continue;
 
-				if ((ask[XARCHIVETYPE_TAR] == xa_tar_ask && ask[i->compressor] == xa_gzip_et_al_ask) ||
-				    (ask[XARCHIVETYPE_TAR] == xa_unar_ask && ask[i->compressor] == xa_unar_ask))
+				if ((archiver[XARCHIVETYPE_TAR].ask == xa_tar_ask && archiver[i->compressor].ask == xa_gzip_et_al_ask) ||
+				    (archiver[XARCHIVETYPE_TAR].ask == xa_unar_ask && archiver[i->compressor].ask == xa_unar_ask))
 				{
 					archiver[type].is_compressor = archiver[i->compressor].is_compressor;
 					archiver[type].type = g_slist_append(archiver[type].type, i->type[0]);
@@ -802,11 +893,11 @@ static void xa_check_available_archivers ()
 					archiver[type].type = g_slist_append(archiver[type].type, i->type[1]);
 					archiver[type].glob = g_slist_append(archiver[type].glob, i->glob[1]);
 
-					ask[type] = ask[XARCHIVETYPE_TAR];
-					list[type] = list[XARCHIVETYPE_TAR];
-					extract[type] = extract[XARCHIVETYPE_TAR];
-					add[type] = add[XARCHIVETYPE_TAR];
-					delete[type] = delete[XARCHIVETYPE_TAR];
+					archiver[type].ask = archiver[XARCHIVETYPE_TAR].ask;
+					archiver[type].list = archiver[XARCHIVETYPE_TAR].list;
+					archiver[type].extract = archiver[XARCHIVETYPE_TAR].extract;
+					archiver[type].add = archiver[XARCHIVETYPE_TAR].add;
+					archiver[type].delete = archiver[XARCHIVETYPE_TAR].delete;
 				}
 			}
 		}
@@ -820,13 +911,14 @@ static XArchive *xa_init_structure_from_cmd_line (char *filename)
 {
 	XArchive *archive;
 	ArchiveType xa;
+	gchar *path;
 
 	xa = xa_detect_archive_type(filename);
 
 	if (xa.type == XARCHIVETYPE_UNKNOWN || xa.type == XARCHIVETYPE_NOT_FOUND)
 		return NULL;
 
-	if (!list[xa.type])
+	if (!archiver[xa.type].list)
 	{
 		xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Sorry, this archive format is not supported:"), _("The proper archiver is not installed!"));
 		return NULL;
@@ -839,21 +931,34 @@ static XArchive *xa_init_structure_from_cmd_line (char *filename)
 		xa_show_message_dialog (NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_("Can't allocate memory for the archive structure!"),"" );
 		return NULL;
 	}
-	archive->path[0] = g_strdup(filename);
-	archive->path[1] = xa_escape_bad_chars(filename, ESCAPES);
-	return (archive);
+
+	if (*filename != '/')
+	{
+		gchar *cur_dir = g_get_current_dir();
+		path = g_strconcat(cur_dir, "/", filename, NULL);
+		g_free(cur_dir);
+	}
+	else
+		path = g_strdup(filename);
+
+	archive->path[0] = g_strdup(path);
+	archive->path[1] = xa_escape_bad_chars(path, ESCAPES);
+
+	g_free(path);
+
+	return archive;
 }
 
 int main (int argc, char **argv)
 {
 	GError *cli_error = NULL;
-	XArchive *archive = NULL;
+	XArchive *archive;
 	gboolean no_bzip2_gzip;
-	unsigned short int x;
-	gchar *_current_dir;
+	gchar *current_dir;
+	int i, result = -1;
 
 #ifdef ENABLE_NLS
-	setlocale(LC_ALL, "");
+	GTK_COMPAT_PARAMETER_STRING;
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
@@ -895,53 +1000,93 @@ int main (int argc, char **argv)
 
 	xdg_open = g_find_program_in_path("xdg-open");
 
-	xa_check_available_archivers();
 	prefs_window   = xa_create_prefs_dialog();
 	extract_window = xa_create_extract_dialog();
 	add_window     = xa_create_add_dialog();
 	multi_extract_window = xa_create_multi_extract_dialog();
-	xa_prefs_load_options(prefs_window);
 
-	if (opt_extract || opt_extract_path || opt_multi_extract || opt_add || opt_compress || opt_info)
+	xa_prefs_load_options(prefs_window);
+	xa_check_available_archivers();
+	xa_prefs_adapt_options(prefs_window);
+
+	if (opt_extract || opt_extract_path || opt_ensure_dir || opt_multi_extract || opt_add || opt_compress || opt_info)
 	{
 		archive = xa_init_structure_from_cmd_line (argv[1]);
 		g_print(PACKAGE_NAME " 0.5.4 \xC2\xA9  " COPYRIGHT_YEAR " " COPYRIGHT_HOLDER "\n");
 		g_print(PACKAGE_NAME " %-8s " MAINTAINER "\n", VERSION);
 
-		/* Switch -x */
-		if (opt_extract_path)
+		/* Switches -d and -x */
+		if (opt_ensure_dir || opt_extract_path)
 		{
+			gchar *extraction_dir = NULL;
+
 			if (!archive)
 			{
 				if (!argv[1])
 					xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't extract files from the archive:"), _("You missed the archive name!\n"));
 
-				return -1;
+				goto leave;
 			}
 
 			if (!archive->can_extract)
 			{
 				xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't extract files from the archive:"), argv[1]);
-				return -1;
+				goto leave;
 			}
 
 			xa_detect_encrypted_archive(archive);
 
 			if (archive->status == XARCHIVESTATUS_ERROR)
-				goto done;
+				goto leave;
 
 			if (archive->has_password)
 			{
 				if (!xa_check_password(archive))
-					goto done;
+					goto leave;
+			}
+
+			if (opt_ensure_dir)
+			{
+				XEntry *entry;
+
+				current_dir = g_get_current_dir();
+				archive->extraction_dir = xa_escape_bad_chars(current_dir, ESCAPES);
+
+				i = 0;
+				entry = archive->root_entry->child;
+
+				while (entry)
+				{
+					if (++i > 1) break;
+
+					entry = entry->next;
+				}
+
+				if ((i != 1) || !archive->root_entry->child->is_dir)
+				{
+					extraction_dir = xa_create_containing_directory(archive, current_dir);
+
+					if (!extraction_dir)
+					{
+						xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't create directory!"), "");
+						g_free(current_dir);
+						goto leave;
+					}
+
+					g_free(archive->extraction_dir);
+					opt_extract_path = extraction_dir;
+				}
+
+				g_free(current_dir);
 			}
 
-			GSList *string = NULL;
 			archive->do_full_path = TRUE;
 			archive->do_overwrite = TRUE;
 			archive->extraction_dir = xa_escape_bad_chars(opt_extract_path, ESCAPES);
+			g_free(extraction_dir);
+
 			archive->status = XARCHIVESTATUS_EXTRACT;
-			(*archive->extract) (archive,string);
+			(*archive->archiver->extract)(archive, NULL);
 		}
 		/* Switch -e */
 		else if (opt_extract)
@@ -951,24 +1096,24 @@ int main (int argc, char **argv)
 				if (!argv[1])
 					xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't extract files from the archive:"), _("You missed the archive name!\n"));
 
-				return -1;
+				goto leave;
 			}
 
 			if (!archive->can_extract)
 			{
 				xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't extract files from the archive:"), argv[1]);
-				return -1;
+				goto leave;
 			}
 
 			xa_detect_encrypted_archive(archive);
 
 			if (archive->status == XARCHIVESTATUS_ERROR)
-				goto done;
+				goto leave;
 
 			if (archive->has_password)
 			{
 				if (!xa_check_password(archive))
-					goto done;
+					goto leave;
 			}
 
 			xa_set_extract_dialog_options(extract_window,0,archive);
@@ -981,9 +1126,9 @@ int main (int argc, char **argv)
 		{
 			Multi_extract_data *multi_extract = NULL;
 			multi_extract = xa_create_multi_extract_dialog();
-			for (x = 1; x< argc; x++)
-				if (! g_file_test(argv[x], G_FILE_TEST_IS_DIR))
-					xa_multi_extract_dialog_add_file(argv[x], multi_extract);
+			for (i = 1; i < argc; i++)
+				if (!g_file_test(argv[i], G_FILE_TEST_IS_DIR))
+					xa_multi_extract_dialog_add_file(argv[i], multi_extract);
 
 			xa_multi_extract_dialog(multi_extract);
 			gtk_widget_destroy (multi_extract->multi_extract);
@@ -998,24 +1143,24 @@ int main (int argc, char **argv)
 				no_bzip2_gzip = FALSE;
 			archive = xa_new_archive_dialog(opt_compress, NULL, no_bzip2_gzip);
 			if (archive == NULL)
-				return -1;
+				goto leave;
 
 			if (archive->path[0] != NULL)
 			{
 				xa_create_working_directory(archive);
-				_current_dir = g_path_get_dirname(opt_compress);
-				chdir (_current_dir);
-				g_free(_current_dir);
+				current_dir = g_path_get_dirname(opt_compress);
+				chdir(current_dir);
+				g_free(current_dir);
 				GSList *files = NULL;
-				_current_dir = g_path_get_basename(opt_compress);
-				files = g_slist_append(files,g_strdup(_current_dir));
-				g_free(_current_dir);
+				current_dir = g_path_get_basename(opt_compress);
+				files = g_slist_append(files, g_strdup(current_dir));
+				g_free(current_dir);
 				g_free(opt_compress);
-				for (x = 1; x< argc; x++)
+				for (i = 1; i < argc; i++)
 				{
-					_current_dir = g_path_get_basename(argv[x]);
-					files = g_slist_append(files,g_strdup(_current_dir));
-					g_free (_current_dir);
+					current_dir = g_path_get_basename(argv[i]);
+					files = g_slist_append(files, g_strdup(current_dir));
+					g_free(current_dir);
 				}
 				xa_execute_add_commands(archive, files, TRUE, NULL);
 			}
@@ -1028,23 +1173,23 @@ int main (int argc, char **argv)
 				if (!argv[1])
 					xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't add files to the archive:"), _("You missed the archive name!\n"));
 
-				return -1;
+				goto leave;
 			}
 			if (!archive->can_add)
 			{
 				xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't add files to the archive:"), argv[1]);
-				return -1;
+				goto leave;
 			}
 
 			xa_detect_encrypted_archive(archive);
 
 			if (archive->status == XARCHIVESTATUS_ERROR)
-				goto done;
+				goto leave;
 
 			if (archive->has_password)
 			{
 				if (!xa_check_password(archive))
-					goto done;
+					goto leave;
 			}
 
 			xa_set_add_dialog_options(add_window, archive);
@@ -1057,9 +1202,9 @@ int main (int argc, char **argv)
 		{
 			GSList *list;
 
-			for (x = XARCHIVETYPE_FIRST; x < XARCHIVETYPE_TYPES; x++)
+			for (i = XARCHIVETYPE_FIRST; i < XARCHIVETYPE_TYPES; i++)
 			{
-				list = archiver[x].type;
+				list = archiver[i].type;
 
 				while (list)
 				{
@@ -1077,15 +1222,19 @@ int main (int argc, char **argv)
 						g_print(", ");
 					else
 					{
-						g_print(":\n  %s\n", archiver[x].program[0]);
+						g_print(":\n  %s\n", archiver[i].program[0]);
 
-						if (archiver[x].program[1])
-							g_print("  %s\n", archiver[x].program[1]);
+						if (archiver[i].program[1])
+							g_print("  %s\n", archiver[i].program[1]);
 					}
 				}
 			}
 		}
-done:
+
+		result = EXIT_SUCCESS;
+
+leave:
+
 		if (progress)
 		{
 			gtk_widget_destroy(progress->window);
@@ -1149,18 +1298,20 @@ done:
 		}
 		#endif
 		gtk_main ();
+
+		result = EXIT_SUCCESS;
 	}
 
-	for (x = XARCHIVETYPE_FIRST; x < XARCHIVETYPE_TYPES; x++)
+	for (i = XARCHIVETYPE_FIRST; i < XARCHIVETYPE_TYPES; i++)
 	{
-		g_free(archiver[x].program[0]);
-		g_free(archiver[x].program[1]);
-		g_slist_free(archiver[x].type);
-		g_slist_free(archiver[x].glob);
-		g_slist_free(archiver[x].version);
+		g_free(archiver[i].program[0]);
+		g_free(archiver[i].program[1]);
+		g_slist_free(archiver[i].type);
+		g_slist_free(archiver[i].glob);
+		g_slist_free(archiver[i].tags);
 	}
 
 	g_free(xdg_open);
 
-	return 0;
+	return result;
 }
