@@ -30,7 +30,7 @@ typedef struct
 
 static GSList *icon_cache;
 
-static gint xa_icon_name_compare_func (pixbuf_cache *a, pixbuf_cache *b)
+static gint xa_icon_name_compare_func (const pixbuf_cache *a, const pixbuf_cache *b)
 {
 	return strcmp(a->icon_name, b->icon_name);
 }
@@ -84,6 +84,8 @@ const char *xa_get_stock_mime_icon(const char *filename)
 	         strcmp(mime,"application/x-cpio-compressed") == 0 ||
 	         strcmp(mime,"application/x-java-archive") == 0 ||
 	         strcmp(mime,"application/x-lha") == 0 ||
+	         strcmp(mime,"application/x-lrzip") == 0 ||
+	         strcmp(mime,"application/x-lrzip-compressed-tar") == 0 ||
 	         strcmp(mime,"application/x-lz4") == 0 ||
 	         strcmp(mime,"application/x-lz4-compressed-tar") == 0 ||
 	         strcmp(mime,"application/x-lzip") == 0 ||
@@ -96,6 +98,7 @@ const char *xa_get_stock_mime_icon(const char *filename)
 	         strcmp(mime,"application/x-tar") == 0 ||
 	         strcmp(mime,"application/x-tarz") == 0 ||
 	         strcmp(mime,"application/x-tzo") == 0 ||
+	         strcmp(mime,"application/x-xpinstall") == 0 ||
 	         strcmp(mime,"application/x-xz") == 0 ||
 	         strcmp(mime,"application/x-xz-compressed-tar") == 0 ||
 	         strcmp(mime,"application/zip") == 0)
@@ -107,7 +110,8 @@ const char *xa_get_stock_mime_icon(const char *filename)
 	         strcmp(mime,"application/x-php") == 0 ||
 	         strcmp(mime,"application/x-shellscript") == 0)
 		return "text-x-script";
-	else if (strncmp(mime,"application/x-font",18) == 0)
+	else if (strncmp(mime,"font/",5) == 0 ||
+	         strncmp(mime,"application/x-font",18) == 0)
 		return "font-x-generic";
 
 	return icon_name;
