@@ -25,6 +25,6 @@
 
 void xa_free_icon_cache();
 GdkPixbuf *xa_get_pixbuf_icon_from_cache(gchar *, gint);
-const char *xa_get_stock_mime_icon(char *);
+const char *xa_get_stock_mime_icon(const char *);
 
 #endif
