@@ -36,7 +36,24 @@ do                                                           \
 }                                                            \
 while (0)
 
-#define USE_PARSER static int _pos = -1; char *_start = line
+#define USE_PARSER         \
+static void *_archive;     \
+static int _pos;           \
+unsigned int _len;         \
+char *_start = line;       \
+                           \
+(void) _len;               \
+                           \
+do                         \
+{                          \
+  if (_archive != archive) \
+  {                        \
+    _archive = archive;    \
+    _pos = -1;             \
+  }                        \
+}                          \
+while (0)
+
 #define NEXT_ITEMS(parts, item) GRAB_ITEMS(parts, item, *line != ' ' && *line != '\t' && *line != '\n')
 #define NEXT_ITEM(item) NEXT_ITEMS(1, item)
 #define SKIP_ITEM NEXT_ITEM(line)
@@ -53,4 +70,7 @@ do                                            \
 }                                             \
 while (0)
 
+#define IF_ITEM_LINE(string) if ((_len = strlen(string)) && (strncmp(line, string, _len) == 0) && (line += _len))
+#define DUPE_ITEM(item) item = g_strdup(g_strstrip(line))
+
 #endif
