@@ -19,6 +19,7 @@
 #include <string.h>
 #include "pref_dialog.h"
 #include "add_dialog.h"
+#include "archive.h"
 #include "extract_dialog.h"
 #include "interface.h"
 #include "main.h"
@@ -27,6 +28,8 @@
 gchar *config_file;
 GtkIconTheme *icon_theme;
 
+static gint preferred_format;
+
 static gchar *xa_prefs_choose_program (gboolean flag)
 {
 	gchar *filename = NULL;
@@ -47,10 +50,10 @@ static gchar *xa_prefs_choose_program (gboolean flag)
 	return filename;
 }
 
-static void xa_prefs_combo_changed (GtkComboBox *widget, gpointer user_data)
+static void xa_prefs_combo_changed (GtkComboBox *widget, gpointer data)
 {
 	gchar *filename, *filename_utf8;
-	unsigned short int flag = GPOINTER_TO_UINT(user_data);
+	unsigned short int flag = GPOINTER_TO_UINT(data);
 
 	if (gtk_combo_box_get_active(GTK_COMBO_BOX (widget)) == 1)
 	{
@@ -70,6 +73,7 @@ static void xa_prefs_combo_changed (GtkComboBox *widget, gpointer user_data)
 static void xa_prefs_dialog_set_default_options (Prefs_dialog_data *prefs_data)
 {
 	gtk_combo_box_set_active (GTK_COMBO_BOX(prefs_data->combo_prefered_format),0);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_data->prefer_unzip), TRUE);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs_data->confirm_deletion),TRUE);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs_data->store_output),FALSE);
 
@@ -91,14 +95,14 @@ static void xa_prefs_dialog_set_default_options (Prefs_dialog_data *prefs_data)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs_data->check_save_geometry),FALSE);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs_data->allow_sub_dir),FALSE);
 	/* Set the default options in the extract dialog */
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(extract_window->ensure_directory), TRUE);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (extract_window->extract_full),TRUE);
 	/* Set the default options in the add dialog */
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (add_window->recurse),TRUE);
 }
 
-void xa_prefs_iconview_changed (GtkIconView *iconview, gpointer data)
+void xa_prefs_iconview_changed (GtkIconView *iconview, Prefs_dialog_data *prefs)
 {
-	Prefs_dialog_data *prefs = data;
 	GList *list;
 	GtkTreePath *path;
 	GtkTreeIter iter;
@@ -196,7 +200,10 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 
 	prefs_data->combo_prefered_format = gtk_combo_box_text_new();
 	gtk_box_pack_start (GTK_BOX (hbox1), prefs_data->combo_prefered_format,FALSE,TRUE,0);
-	xa_combo_box_text_append_compressor_types(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_format));
+
+	prefs_data->prefer_unzip = gtk_check_button_new_with_mnemonic(_("Prefer unzip for zip files (requires restart)"));
+	gtk_box_pack_start(GTK_BOX(vbox4), prefs_data->prefer_unzip, FALSE, FALSE, 0);
+	gtk_button_set_focus_on_click(GTK_BUTTON(prefs_data->prefer_unzip), FALSE);
 
 	prefs_data->confirm_deletion = gtk_check_button_new_with_mnemonic (_("Confirm deletion of files"));
 	gtk_box_pack_start (GTK_BOX (vbox4), prefs_data->confirm_deletion, FALSE, FALSE, 0);
@@ -397,6 +404,7 @@ void xa_prefs_save_options(Prefs_dialog_data *prefs_data, const char *filename)
 	GKeyFile *xa_key_file = g_key_file_new();
 
 	g_key_file_set_integer (xa_key_file,PACKAGE,"preferred_format",gtk_combo_box_get_active (GTK_COMBO_BOX(prefs_data->combo_prefered_format)));
+	g_key_file_set_boolean(xa_key_file, PACKAGE, "prefer_unzip", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_data->prefer_unzip)));
 	g_key_file_set_boolean (xa_key_file,PACKAGE,"confirm_deletion",gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (prefs_data->confirm_deletion)));
 	g_key_file_set_boolean (xa_key_file,PACKAGE,"sort_filename_content",gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (prefs_data->check_sort_filename_column)));
 	g_key_file_set_boolean (xa_key_file,PACKAGE,"store_output",gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (prefs_data->store_output)));
@@ -473,6 +481,7 @@ void xa_prefs_save_options(Prefs_dialog_data *prefs_data, const char *filename)
 		g_key_file_set_integer_list(xa_key_file, PACKAGE, "add", prefs_data->add_coords,2);
 	}
 	/* Save the options in the extract dialog */
+	g_key_file_set_boolean(xa_key_file, PACKAGE, "ensure_directory", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(extract_window->ensure_directory)));
 	g_key_file_set_boolean (xa_key_file,PACKAGE,"overwrite",gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (extract_window->overwrite_check)));
 	g_key_file_set_boolean (xa_key_file,PACKAGE,"full_path",gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (extract_window->extract_full)));
 	g_key_file_set_boolean (xa_key_file,PACKAGE,"touch",    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (extract_window->touch)));
@@ -503,7 +512,7 @@ void xa_prefs_load_options(Prefs_dialog_data *prefs_data)
 	gchar *xarchiver_config_dir = NULL;
 	GKeyFile *xa_key_file = g_key_file_new();
 	GError *error = NULL;
-	gboolean toolbar;
+	gboolean unzip, toolbar;
 
 	config_dir = g_strconcat (g_get_home_dir(),"/.config",NULL);
 	if (g_file_test(config_dir, G_FILE_TEST_EXISTS) == FALSE)
@@ -525,7 +534,20 @@ void xa_prefs_load_options(Prefs_dialog_data *prefs_data)
 	}
 	else
 	{
-		gtk_combo_box_set_active (GTK_COMBO_BOX(prefs_data->combo_prefered_format),g_key_file_get_integer(xa_key_file,PACKAGE,"preferred_format",NULL));
+		preferred_format = g_key_file_get_integer(xa_key_file, PACKAGE, "preferred_format", NULL);
+		unzip = g_key_file_get_boolean(xa_key_file, PACKAGE, "prefer_unzip", &error);
+
+		if (error)
+		{
+			if (error->code == G_KEY_FILE_ERROR_KEY_NOT_FOUND)
+				/* preserve default setting with existing, old config file */
+				unzip = TRUE;
+
+			g_error_free(error);
+			error = NULL;
+		}
+
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_data->prefer_unzip), unzip);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs_data->confirm_deletion),g_key_file_get_boolean(xa_key_file,PACKAGE,"confirm_deletion",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs_data->store_output),g_key_file_get_boolean(xa_key_file,PACKAGE,"store_output",NULL));
 
@@ -538,7 +560,7 @@ void xa_prefs_load_options(Prefs_dialog_data *prefs_data)
 		if (error)
 		{
 			if (error->code == G_KEY_FILE_ERROR_KEY_NOT_FOUND)
-				/* Preserve toolbar with old config files */
+				/* preserve default setting with existing, old config file */
 				toolbar = TRUE;
 
 			g_error_free(error);
@@ -656,6 +678,7 @@ void xa_prefs_load_options(Prefs_dialog_data *prefs_data)
 			prefs_data->add_coords[1] = add_coords[1];
 		}
 		/* Load the options in the extract dialog */
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(extract_window->ensure_directory), g_key_file_get_boolean(xa_key_file, PACKAGE, "ensure_directory", NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(extract_window->overwrite_check),g_key_file_get_boolean(xa_key_file,PACKAGE,"overwrite",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(extract_window->extract_full),g_key_file_get_boolean(xa_key_file,PACKAGE,"full_path",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(extract_window->touch),g_key_file_get_boolean(xa_key_file,PACKAGE,"touch",NULL));
@@ -673,7 +696,13 @@ void xa_prefs_load_options(Prefs_dialog_data *prefs_data)
 	/* config_file is freed in window.c xa_quit_application */
 }
 
-void xa_apply_prefs_option(Prefs_dialog_data *prefs_data)
+void xa_prefs_adapt_options (Prefs_dialog_data *prefs_data)
+{
+	xa_combo_box_text_append_compressor_types(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_format));
+	gtk_combo_box_set_active(GTK_COMBO_BOX(prefs_data->combo_prefered_format), preferred_format);
+}
+
+void xa_prefs_apply_options (Prefs_dialog_data *prefs_data)
 {
 	gint i,idx;
 
