@@ -19,17 +19,15 @@
 #ifndef RAR_H
 #define RAR_H
 
-#include <gtk/gtk.h>
-#include <string.h>
-#include <stdlib.h>
-#include "string_utils.h"
-#include "support.h"
+#include <glib.h>
 #include "archive.h"
-gboolean jump_header,jump_comment,read_filename, last_line, encrypted;
-void xa_rar_delete (XArchive *,GSList *);
-void xa_rar_add (XArchive *,GString *,gchar *);
-gboolean xa_rar_extract(XArchive *,GSList *);
-void xa_rar_test (XArchive *);
-void xa_get_rar_line_content (gchar *line, gpointer data);
-void xa_open_rar ( XArchive *archive );
+
+void xa_rar_add(XArchive *, GSList *, gchar *);
+void xa_rar_ask(XArchive *);
+int xa_rar_check_version(gchar *);
+void xa_rar_delete(XArchive *, GSList *);
+gboolean xa_rar_extract(XArchive *, GSList *);
+void xa_rar_list(XArchive *);
+void xa_rar_test(XArchive *);
+
 #endif
