@@ -28,7 +28,6 @@
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
-#include <locale.h>
 #define gettext_noop(String) String
 #define _(String) dgettext(GETTEXT_PACKAGE, String)
 #define N_(String) gettext_noop(String)
@@ -158,12 +157,21 @@ static inline void gtk_combo_box_text_remove (GtkWidget *combo_box, gint positio
 #define GTK_COMPAT_ABOUT_DIALOG_URI(about, func) g_signal_connect(about, "activate-link", G_CALLBACK(func), NULL)
 #endif
 
+#if !GTK_CHECK_VERSION(2,24,32)
+#include <locale.h>
+#define GTK_COMPAT_PARAMETER_STRING setlocale(LC_ALL, "")
+#else
+#define GTK_COMPAT_PARAMETER_STRING (void) 0
+#endif
+
 #if GTK_CHECK_VERSION(3,0,0)
 #define GTK_COMPAT_ADJUSTMENT_TYPE GtkAdjustment *
+#define GTK_COMPAT_DESTROY_TYPE GtkWidget *
 #undef GTK_COMPAT_SWITCH_PAGE_TYPE
 #define GTK_COMPAT_SWITCH_PAGE_TYPE GtkWidget *
 #else
 #define GTK_COMPAT_ADJUSTMENT_TYPE GtkObject *
+#define GTK_COMPAT_DESTROY_TYPE GtkObject *
 #endif
 
 #endif
