@@ -18,6 +18,7 @@
 
 #include <string.h>
 #include "tar.h"
+#include "gzip_et_al.h"
 #include "main.h"
 #include "string_utils.h"
 #include "support.h"
@@ -26,21 +27,32 @@
 gboolean isTar (FILE *file)
 {
 	unsigned char magic[8];
+	gboolean result;
 
 	fseek(file, 0, SEEK_SET);
 
 	if (fseek(file, 257, SEEK_CUR) != 0 ||
-	    fread(magic, 1, sizeof(magic), file) != sizeof(magic))
+	    fread(magic, sizeof(magic), 1, file) != 1)
 	{
 		fseek(file, 0, SEEK_SET);
 		return FALSE;
 	}
 
+	result = (memcmp(magic, "ustar" "\x00" "00", sizeof(magic)) == 0 ||
+	          memcmp(magic, "ustar" "  " "\x00", sizeof(magic)) == 0);
+
+	if (!result &&
+	    memcmp(magic, "\x00\x00\x00\x00\x00\x00\x00\x00", sizeof(magic)) == 0)
+	{
+		/* next block */
+		if (fseek(file, 512, SEEK_SET) == 0 &&
+		    fread(magic, sizeof(*magic), 1, file) == 1)
+			result = (*magic != 0);
+	}
+
 	fseek(file, 0, SEEK_SET);
 
-	return (memcmp(magic, "\x75\x73\x74\x61\x72\x00\x30\x30", sizeof(magic)) == 0 ||
-	        memcmp(magic, "\x75\x73\x74\x61\x72\x20\x20\x00", sizeof(magic)) == 0 ||
-	        memcmp(magic, "\x00\x00\x00\x00\x00\x00\x00\x00", sizeof(magic)) == 0);
+	return result;
 }
 
 static XArchiveType xa_tar_get_compressor_type (XArchive *archive)
@@ -53,6 +65,9 @@ static XArchiveType xa_tar_get_compressor_type (XArchive *archive)
 		case XARCHIVETYPE_TAR_GZ:
 			return XARCHIVETYPE_GZIP;
 
+		case XARCHIVETYPE_TAR_LRZ:
+			return XARCHIVETYPE_LRZIP;
+
 		case XARCHIVETYPE_TAR_LZ:
 			return XARCHIVETYPE_LZIP;
 
@@ -71,6 +86,9 @@ static XArchiveType xa_tar_get_compressor_type (XArchive *archive)
 		case XARCHIVETYPE_TAR_Z:
 			return XARCHIVETYPE_COMPRESS;
 
+		case XARCHIVETYPE_TAR_ZSTD:
+			return XARCHIVETYPE_ZSTD;
+
 		default:
 			return XARCHIVETYPE_TAR;
 	}
@@ -178,6 +196,8 @@ static void xa_tar_parse_output (gchar *line, XArchive *archive)
 
 	if (entry)
 	{
+		entry->is_encrypted = (archive->password != NULL);
+
 		if (!entry->is_dir)
 			archive->files++;
 
@@ -282,7 +302,7 @@ gboolean xa_tar_extract (XArchive *archive, GSList *file_list)
 void xa_tar_add (XArchive *archive, GSList *file_list, gchar *compression)
 {
 	GString *files;
-	gchar *command, *workfile, *archive_path;
+	gchar *command;
 
 	if (archive->location_path != NULL)
 		archive->child_dir = g_strdup(archive->working_dir);
@@ -295,6 +315,8 @@ void xa_tar_add (XArchive *archive, GSList *file_list, gchar *compression)
 			archive->path[2] = g_shell_quote(archive->path[0]);
 		else
 		{
+			gchar *workfile;
+
 			if (!xa_create_working_directory(archive))
 			{
 				g_string_free(files, TRUE);
@@ -323,11 +345,7 @@ void xa_tar_add (XArchive *archive, GSList *file_list, gchar *compression)
 		xa_run_command(archive, command);
 		g_free(command);
 
-		workfile = xa_escape_bad_chars(archive->path[2], "\"");
-		archive_path = xa_quote_shell_command(archive->path[0], TRUE);
-		command = g_strconcat("sh -c \"", archiver[xa_tar_get_compressor_type(archive)].program[0], " ", workfile, " -c > ", archive_path, "\"", NULL);
-		g_free(archive_path);
-		g_free(workfile);
+		command = xa_gzip_et_al_get_command(archiver[xa_tar_get_compressor_type(archive)].program[0], archive->path[2], archive->path[0], archive->password, xa_tar_get_compressor_type(archive));
 	}
 
 	g_string_free(files, TRUE);
@@ -339,7 +357,7 @@ void xa_tar_add (XArchive *archive, GSList *file_list, gchar *compression)
 void xa_tar_delete (XArchive *archive, GSList *file_list)
 {
 	GString *files;
-	gchar *command, *workfile, *archive_path;
+	gchar *command;
 
 	files = xa_quote_filenames(file_list, NULL, TRUE);
 	command = g_strconcat(archiver[XARCHIVETYPE_TAR].program[0], " --delete --no-recursion --no-wildcards -f ", archive->path[2], files->str, NULL);
@@ -349,11 +367,7 @@ void xa_tar_delete (XArchive *archive, GSList *file_list)
 		xa_run_command(archive, command);
 		g_free(command);
 
-		workfile = xa_escape_bad_chars(archive->path[2], "\"");
-		archive_path = xa_quote_shell_command(archive->path[0], TRUE);
-		command = g_strconcat("sh -c \"", archiver[xa_tar_get_compressor_type(archive)].program[0], " ", workfile, " -c > ", archive_path, "\"", NULL);
-		g_free(archive_path);
-		g_free(workfile);
+		command = xa_gzip_et_al_get_command(archiver[xa_tar_get_compressor_type(archive)].program[0], archive->path[2], archive->path[0], archive->password, xa_tar_get_compressor_type(archive));
 	}
 
 	g_string_free(files, TRUE);
