@@ -34,11 +34,11 @@ void xa_add_files_archive(GtkMenuItem *, gpointer);
 void xa_archive_properties(GtkMenuItem *, gpointer);
 void xa_cancel_archive(GtkMenuItem *, gpointer);
 void xa_child_processed(XAChildProcess, gboolean, XArchive *);
-void xa_clipboard_clear(GtkClipboard *, gpointer);
+void xa_clipboard_clear(GtkClipboard *, XArchive *);
 void xa_clipboard_copy(GtkMenuItem *, gpointer);
 void xa_clipboard_cut(GtkMenuItem *, gpointer);
 void xa_clipboard_paste(GtkMenuItem *, gpointer);
-void xa_close_archive(GtkMenuItem *, gpointer);
+void xa_close_archive(GtkWidget *, gpointer);
 void xa_concat_selected_filenames(GtkTreeModel *, GtkTreePath *, GtkTreeIter *, GSList **);
 void xa_convert_sfx(GtkMenuItem *, gpointer);
 void xa_create_liststore(XArchive *, const gchar *[]);
@@ -53,7 +53,7 @@ void xa_location_entry_activated(GtkEntry *, gpointer);
 GtkWidget *xa_main_window_find_image(gchar *, GtkIconSize);
 gboolean xa_mouse_button_event(GtkWidget *, GdkEventButton *, XArchive *);
 void xa_new_archive(GtkMenuItem *, gpointer);
-void xa_open_archive(GtkMenuItem *, gpointer);
+void xa_open_archive(GtkWidget *, gchar *);
 void xa_open_with_from_popupmenu(GtkMenuItem *, gpointer);
 void xa_quit_application(GtkWidget *, GdkEvent *, gpointer);
 void xa_reload_archive_content(XArchive *);
