@@ -37,6 +37,7 @@ void xa_zip_ask (XArchive *archive)
 	archive->can_password = TRUE;
 	archive->can_full_path[0] = TRUE;
 	archive->can_full_path[1] = archiver[archive->type].is_compressor;
+	archive->can_touch = TRUE;
 	archive->can_overwrite = TRUE;
 	archive->can_update[0] = TRUE;
 	archive->can_update[1] = archiver[archive->type].is_compressor;
@@ -60,16 +61,15 @@ static void xa_zip_parse_output (gchar *line, XArchive *archive)
 	XEntry *entry = NULL;
 
 	gchar *filename;
-	gpointer item[8];
-	unsigned short int i = 0;
+	gpointer item[9];
 	unsigned int linesize,n,a;
 	gboolean encrypted,dir;
+	guint64 size;
 
 	encrypted = dir = FALSE;
-	if ((line[0] != 'd') && (line[0] != '-') && (line[0] != '?'))
+	if ((line[0] != '-') && (line[0] != 'd') && (line[0] != 'l') && (line[0] != '?'))
 		return;
 
-	archive->files++;
 	linesize = strlen(line);
 
 	/* permissions */
@@ -78,10 +78,9 @@ static void xa_zip_parse_output (gchar *line, XArchive *archive)
 	for(; n < linesize && line[n] != ' '; n++);
 
 	line[n]='\0';
-	item[i] = line + a;
+	item[5] = line + a;
 	if ( (line+a)[0] == 'd')
 		dir = TRUE;
-	i++;
 	n++;
 
 	/* version */
@@ -90,8 +89,7 @@ static void xa_zip_parse_output (gchar *line, XArchive *archive)
 	for(; n < linesize && line[n] != ' '; n++);
 
 	line[n] = '\0';
-	item[i] = line + a;
-	i++;
+	item[8] = line + a;
 	n++;
 
 	/* OS */
@@ -100,8 +98,7 @@ static void xa_zip_parse_output (gchar *line, XArchive *archive)
 	for(; n < linesize && line[n] != ' '; n++);
 
 	line[n]='\0';
-	item[i] = line + a;
-	i++;
+	item[7] = line + a;
 	n++;
 
 	/* size */
@@ -110,12 +107,11 @@ static void xa_zip_parse_output (gchar *line, XArchive *archive)
 	for(; n < linesize && line[n] != ' '; n++);
 
 	line[n]='\0';
-	item[i] = line + a;
-	archive->files_size += g_ascii_strtoull(item[i],NULL,0);
-	i++;
+	item[0] = line + a;
+	size = g_ascii_strtoull(item[0], NULL, 0);
 	n++;
 
-	/* tx/bx */
+	/* text/binary */
 	for(; n < linesize && line[n] == ' '; n++);
 	a = n;
 	for(; n < linesize && line[n] != ' '; n++);
@@ -134,8 +130,7 @@ static void xa_zip_parse_output (gchar *line, XArchive *archive)
 	for(; n < linesize && line[n] != ' '; n++);
 
 	line[n]='\0';
-	item[i] = line + a;
-	i++;
+	item[1] = line + a;
 	n++;
 
 	/* method */
@@ -144,8 +139,7 @@ static void xa_zip_parse_output (gchar *line, XArchive *archive)
 	for(; n < linesize && line[n] != ' '; n++);
 
 	line[n]='\0';
-	item[i] = line + a;
-	i++;
+	item[6] = line + a;
 	n++;
 
 	/* date */
@@ -154,8 +148,7 @@ static void xa_zip_parse_output (gchar *line, XArchive *archive)
 	for(; n < linesize && line[n] != ' '; n++);
 
 	line[n]='\0';
-	item[i] = line + a;
-	i++;
+	item[3] = line + a;
 	n++;
 
 	/* time */
@@ -164,26 +157,41 @@ static void xa_zip_parse_output (gchar *line, XArchive *archive)
 	for(; n < linesize && line[n] != ' '; n++);
 
 	line[n]='\0';
-	item[i] = line + a;
+	item[4] = line + a;
 	n++;
 
 	/* filename */
 	line[linesize-1] = '\0';
 	filename = line + n;
 
-	entry = xa_set_archive_entries_for_each_row (archive,filename,item);
-	if (entry != NULL)
+	/* saving */
+	if (size)
+		item[2] = g_strdup_printf("%.1f%%", 100.0 - 100.0 * g_ascii_strtoull(item[1], NULL, 0) / size);
+	else
+		item[2] = g_strdup("-");
+
+	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+
+	if (entry)
 	{
 		if (dir)
 			 entry->is_dir = TRUE;
+
 		entry->is_encrypted = encrypted;
+
+		if (!entry->is_dir)
+			archive->files++;
+
+		archive->files_size += size;
 	}
+
+	g_free(item[2]);
 }
 
 void xa_zip_list (XArchive *archive)
 {
-	const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
-	const gchar *titles[] = {_("Permissions"), _("Version"), _("OS"), _("Original"), _("Compressed"), _("Method"), _("Date"), _("Time")};
+	const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
+	const gchar *titles[] = {_("Original Size"), _("Compressed"), _("Saving"), _("Date"), _("Time"), _("Permissions"), _("Method"), _("OS"), _("Version")};
 	guint i;
 
 	gchar *command = g_strconcat(archiver[archive->type].program[0], " -Z -l ", archive->path[1], NULL);
@@ -193,7 +201,8 @@ void xa_zip_list (XArchive *archive)
 	xa_spawn_async_process (archive,command);
 	g_free ( command );
 
-	archive->columns = 11;
+	archive->columns = 12;
+	archive->size_column = 2;
 	archive->column_types = g_malloc0(sizeof(types));
 
 	for (i = 0; i < archive->columns; i++)
@@ -224,6 +233,7 @@ gboolean xa_zip_extract (XArchive *archive, GSList *file_list)
 	password_str = xa_zip_password_str(archive);
 	command = g_strconcat(archiver[archive->type].program[0],
 	                      archive->do_full_path ? "" : " -j",
+	                      archive->do_touch ? " -DD" : "",
 	                      archive->do_overwrite ? " -o" : " -n",
 	                      archive->do_update ? " -u" : "",
 	                      archive->do_freshen ? " -f" : "",
