@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005 Giuseppe Torelli - <colossus73@gmail.com>
+ *  Copyright (C) 2008 Giuseppe Torelli - <colossus73@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,11 +19,14 @@
 #ifndef ZIP_H
 #define ZIP_H
 
-#include <gtk/gtk.h>
-#include "interface.h"
-#include "callbacks.h"
-#include "main.h"
+#include <glib.h>
 #include "archive.h"
 
-void OpenZip ( XArchive *archive );
+void xa_zip_add(XArchive *, GSList *, gchar *);
+void xa_zip_ask(XArchive *);
+void xa_zip_delete(XArchive *, GSList *);
+gboolean xa_zip_extract(XArchive *, GSList *);
+void xa_zip_list(XArchive *);
+void xa_zip_test(XArchive *);
+
 #endif
